clear all;
close all;
clc;

%% Find the current folder
current_file = mfilename('fullpath');
if isempty(current_file)
    current_file = which('plot_ARres.m');
end
current_folder = fileparts(current_file);


%% Load data
save_file = @(n)(sprintf('%s_%d.mat', fullfile(current_folder, 'AR_save'), round(abs(n))));
nsave = [3, 4, 5];

cost_lev = [];
res = [];
for ks = 1:length(nsave)
    data = load(save_file(nsave(ks)));
    
    if isempty(res)
        res = data.all_res;
    else
        res = cat(1, res, data.all_res);
    end
    if isempty(cost_lev)
        cost_lev = data.cost_lev;
    elseif ~isequal(cost_lev, data.cost_lev)
        stk_error('Different level cost.', 'IncompatibleResults');
    end
    cost_HF = max(cost_lev);
    nb_lev = length(cost_lev);
end

nb_exp = size(res, 1);
nb_alg = size(res, 2);
type_res = 3;

%% Find bounds

% Bounds
limits = NaN(2, 4);

for ksa = 1:nb_alg
    % Load all results
    for ex = 1:nb_exp
        res_ke = res(ex, ksa).results;
        % Find the cost bounds
        limits(1, :) = min(limits(1, :), min(res_ke));
        limits(2, :) = max(limits(2, :), max(res_ke));
        
    end
    
end

%% Load individual error for all cost
lim_cost = stk_hrect([floor(limits(1, 1)); ceil(limits(2, 1))], {'cost'});
cost_ax = lim_cost(1):lim_cost(2);
nb_ax = length(cost_ax);

error_indiv = struct('cost', cost_ax, 'L2_error', [], 'bias', [], 'variance', []);
error_indiv = repmat(error_indiv, nb_alg, nb_exp);

for ksa = 1:nb_alg
    % Load all results
    for ex = 1:nb_exp
        res_ke = res(ex, ksa).results;
        L2 = NaN(nb_ax, 1);
        bi = NaN(nb_ax, 1);
        va = NaN(nb_ax, 1);
        
        for jc = 1:nb_ax
            idx_c = find(res_ke.cost <= cost_ax(jc), 1, 'last');
            L2(jc, 1) = res_ke.error_L2(idx_c);
            bi(jc, 1) = res_ke.bias(idx_c);
            va(jc, 1) = res_ke.variance(idx_c);
        end
        
        error_indiv(ksa, ex).L2_error = L2;
        error_indiv(ksa, ex).bias     = bi;
        error_indiv(ksa, ex).variance = va;
    end
    
end

%% Style
grey = @(x)(repmat(min(max(x(:), 0), 1), 1, 3));

colors = [0, 0, 0.5;
    0.5, 0, 0;
    0.5, 0, 0.5];
markers = {'o', 's', 'd', 'v', 'p', 'h', 'x', '+'};

style_kri = {'LineStyle',  '-', 'LineWidth', 2, 'Marker', 'none'};

ftz_axi = 20;
ftz_lab = 24;
ftz_tit = 28;
ftz_leg = 28;

kunc = -sqrt(2)*erfcinv(1 + 0.95);

% Saving folder
foldersave = ''; % fullfile(pwd, '..', 'redactions', 'rapports', 'these', 'images', 'plan_seq');

%% Plot results


for typ = 1:type_res
    
    if typ == 1
        fig = stk_figure('L2-Error on the probability function');
        field_typ = 'L2_error';
        ylab = '||p_n(x, \delta^{HF}) - p*(x, \delta^{HF})||_2';
        tit = 'Erreur en norme L^2';
        
        fadapt = @sqrt;
        
    elseif typ == 2
        fig = stk_figure('Error on the global probability');
        field_typ = 'bias';
        ylab = '|P - P*|';
        tit = 'Global error';
        
        fadapt = @abs;
        
    elseif typ == 3
        fig = stk_figure('Variance of the probability function');
        field_typ = 'variance';
        ylab = '||Std(p(x, tHF)||';
        tit = 'Variance';
        
        fadapt = @sqrt;
        
    else
        stk_error('Unknown case type of result.', 'UnknownCase')
    end
    set(fig, 'PaperPositionMode', 'auto');
    hold on;
    
    for seq = 1:nb_alg
        %% Get result
        err_typ = fadapt(double([error_indiv(seq, :).(field_typ)]));
        
        %% Get value of the results
        if all(~isnan(err_typ(:)))
            err_med = median(err_typ, 2);
            err_fquart = quantile(err_typ, 0.25, 2);
            err_tquart = quantile(err_typ, 0.75, 2);
        else
            err_med     = NaN(nb_ax, 1);
            err_fquart = NaN(nb_ax, 1);
            err_tquart = NaN(nb_ax, 1);
            for bbc = 1:nb_ax
                err_typ_b = err_typ(bbc, :);
                err_typ_b = err_typ_b(~isnan(err_typ_b));
                
                err_med(bbc, 1) = median(err_typ_b, 2);
                err_fquart(bbc, 1) = quantile(err_typ_b, 0.25, 2);
                err_tquart(bbc, 1) = quantile(err_typ_b, 0.75, 2);
            end
        end
        
        %% Plot results
        cost_plot = double(cost_ax)/cost_HF;
        if seq < nb_alg
            idx_cp = 1:cost_lev(seq):nb_ax;
        else % MRSUR
            idx_cp = 1:1:nb_ax;
        end
        
        plot(cost_plot(idx_cp), err_med(idx_cp), 'Color', colors(seq, :), style_kri{:},...
            'MarkerFaceColor', colors(seq, :), 'Marker', markers{seq});
        % plot(cost_plot(idx_cp), err_fquart(idx_cp),...
        %   'Color', colors(seq, :), style_unc{:}, 'Marker', markers{seq});
        % plot(cost_plot(idx_cp), err_tquart(idx_cp),...
        %   'Color', colors(seq, :), style_unc{:}, 'Marker', markers{seq});
        
    end
    set(get(fig, 'CurrentAxes'), 'FontSize', ftz_axi);
    stk_xlabel ('Cout', 'FontSize', ftz_lab);
    stk_ylabel (ylab,   'FontSize', ftz_lab);
    stk_title  (tit,    'FontSize', ftz_tit);
    xlim(lim_cost/cost_HF);
    ylim([0; max(fadapt(limits(:, typ + 1)))]);
    
    save_fig(foldersave, ['ex1_', field_typ], fig);
end

%% Legend
% Legend
fig = stk_figure('Legend');
set(fig, 'PaperPositionMode', 'auto');

% Inputs for legends
han = NaN(nb_alg, 1);
lab = cell(nb_alg, 1);

hold on;
for seq = 1:nb_alg
    han(seq) = plot([0; 1], [0; 1], 'Color', colors(seq, :), style_kri{:},...
        'MarkerFaceColor', colors(seq, :), 'Marker', markers{seq});
    lab{seq} = ['SUR au niveau ', num2str(seq)];
end
lab{nb_alg} = 'MRSUR';

% Adapt bounds
xlim([-2; -1]);
ylim([-2; -1]);
set(get(fig, 'CurrentAxes'), 'XTick', [], 'YTick', [], 'XColor', 'w', 'YColor', 'w');
leg = legend(han, lab{:});
% Legend orientation
set(leg, 'Orientation', 'Vertical', 'box', 'on', 'FontSize', ftz_leg);
pos_leg = get(leg, 'Position');
pos_leg(1) = 0.5 - pos_leg(3)/2;
pos_leg(2) = 0.5 - pos_leg(4)/2;
set(leg, 'Position', pos_leg);
save_fig(foldersave, 'ex1_legend', fig);

%% Compare MRSUR and SUR last kevek

MRSUR_best = zeros(nb_ax, type_res);
MRSUR_stritly_best = zeros(nb_ax, type_res);

for kex = 1:nb_exp
    results_MRSUR = res(kex, end).results;
    results_SUR_last = res(kex, end - 1).results;
    
    cost_MRSUR = results_MRSUR.cost;
    cost_SUR_last = results_SUR_last.cost;
    
    results_MRSUR_allcost = NaN(nb_ax, type_res);
    results_SURlast_allcost = NaN(nb_ax, type_res);
    
    for jax = 1:nb_ax
        % MRSUR
        idx_cost = find(cost_MRSUR <= cost_ax(jax), 1, 'last');
        results_MRSUR_allcost(jax, :) = results_MRSUR(idx_cost, 2:end);
        
        % SUR second level
        idx_cost = find(cost_SUR_last <= cost_ax(jax), 1, 'last');
        results_SURlast_allcost(jax, :) = results_MRSUR(idx_cost, 2:end);
    end
    % Compare positive values
    results_MRSUR_allcost = abs(results_MRSUR_allcost);
    results_SURlast_allcost = abs(results_SURlast_allcost);
    
    MRSUR_best         = MRSUR_best         + double(results_MRSUR_allcost <= results_SURlast_allcost);
    MRSUR_stritly_best = MRSUR_stritly_best + double(results_MRSUR_allcost < results_SURlast_allcost);
end


idx_sur_last = 1:cost_HF:nb_ax;

fig = stk_figure('Best strategy as function of the cost');
set(fig, 'PaperPositionMode', 'auto');
hold on;
plot(cost_plot(idx_sur_last), MRSUR_stritly_best(idx_sur_last, 1)/nb_exp,...
    '-o', 'Color', 'b', 'LineWidth', 2, 'MarkerFaceColor', 'b');
plot(cost_plot(idx_sur_last), MRSUR_best(idx_sur_last, 1)/nb_exp,...
    '-.d', 'Color', 'r', 'LineWidth', 2);
plot(double(lim_cost)/cost_HF, [0.5, 0.5], 'k-.');
plot(double(lim_cost)/cost_HF, [1, 1], 'k-.');
set(get(fig, 'CurrentAxes'), 'FontSize', ftz_axi);
xlim(double(lim_cost)/cost_HF);
ylim([0; 1]);
stk_xlabel('Cost', 'FontSize', ftz_lab);
stk_ylabel('Proportion', 'FontSize', ftz_lab);
stk_title('MRSUR better than SUR', 'FontSize', ftz_tit);
save_fig(foldersave, 'ex1_MRSUR_best', fig);

%%
cost_all  = [];
level_obs = [];


order_obs = cell(nb_exp, 1);
nb_obs_lev = NaN(nb_exp, nb_lev);

for ka = 1:nb_exp
    % Cost
    cost_ka = res(ka, end).results.cost;
    
    % Observed level (1:S)
    diff_cost_k = diff(cost_ka);
    level_observed = NaN(size(diff_cost_k));
    for tl = 1:nb_lev
        idx_co = (diff_cost_k == cost_lev(tl));
        level_observed(idx_co) = tl;
        nb_obs_lev(ka, tl) = sum(idx_co);
    end
    
    cost_ka = cost_ka(2:end);
    siz_a = size(cost_ka, 1);
    siz_l = size(cost_all, 1);
    if siz_l == siz_a
        cost_all  = cat(2,  cost_all, cost_ka);
        level_obs = cat(2, level_obs, level_observed);
    elseif siz_l > siz_a
        cost_all  = cat(2,  cost_all, [cost_ka; NaN(abs(siz_l - siz_a), 1)]);
        level_obs = cat(2, level_obs, [level_observed; NaN(abs(siz_l - siz_a), 1)]);
    else % siz_l < siz_a
        cost_all  = cat(2,  [cost_all; NaN(abs(siz_a - siz_l), ka - 1)], cost_ka);
        level_obs = cat(2, [level_obs; NaN(abs(siz_a - siz_l), ka - 1)], level_observed);
    end
    
    
    order_obs{ka} = level_observed;
end

%% Proportion of case where the last level is observed

nb_obs_lev_last = nb_obs_lev(:, end);
%Number of times where the last level is selected

budget = (nb_ax - 1);
normal_budget = budget/cost_HF;
nb_sel = normal_budget + 1;
class_2sel = 0:normal_budget;
prop_selection = histc(nb_obs_lev_last, class_2sel)/nb_exp;
prop_selection = prop_selection';
unc_probability = sqrt(prop_selection.*prop_selection/nb_exp);

unc_stem = [prop_selection - 2*unc_probability;
    prop_selection + 2*unc_probability;
    NaN(1, nb_sel);];
kobs_stem = [class_2sel; class_2sel; NaN(1, nb_sel)];

fig = stk_figure('Histogram of selected levels');
set(fig, 'PaperPositionMode', 'auto');
hold on;
%grid on;
plot(kobs_stem, unc_stem, '-+', 'Color', 'b', 'LineWidth', 2);
plot(class_2sel, prop_selection, 'ro', 'LineWidth', 2, 'MarkerFaceColor', 'r');


xtick_new = cell(nb_sel, 1);
for k = class_2sel
    if nb_lev == 2
        xtick_new{k + 1, 1} = sprintf('%d|%d', budget - cost_HF*k, k);
    else
        xtick_new{k + 1, 1} = sprintf('%d', k);
    end
end
set(get(fig, 'CurrentAxes'), 'XTick', class_2sel, 'XTickLabel', class_2sel);

set(get(fig, 'CurrentAxes'), 'FontSize', ftz_axi);
stk_xlabel('Nombre d''observations au niveau 2', 'FontSize', ftz_lab);
stk_ylabel('Proportion de repetitions', 'FontSize', ftz_lab);
stk_title('Compromis entre les niveaux', 'FontSize', ftz_tit);
xlim([min(class_2sel); max(class_2sel)] + 0.2*[-1; 1]);
ylim([0, ceil(max(unc_stem(:))/0.1)*0.1]);
save_fig(foldersave, 'ex1_hist_obs', fig);

%%
color_lev = parula(nb_lev);

levcost = NaN(nb_ax, nb_exp);

fig = stk_figure('Observed levels');
set(fig, 'PaperPositionMode', 'auto');
hold on;
for je = 1:nb_exp
    list_obs_j = order_obs{je};
    list_cost = [0, cost_lev(list_obs_j)];
    cost_all = (lim_cost(1) + cumsum(list_cost))/cost_HF;
    cost_all = cost_all(1:(end - 1));
    
    for ll = 1:nb_lev
        ind_l = (list_obs_j == ll);
        % plot(cost_all(ind_l), je*ones(sum(ind_l), 1), 'Color', 'k',...
        %    'LineStyle', 'none', 'Marker', marker_lev{ll}, 'LineWidth', 2);
        cost_all_ind = cost_all(ind_l);
        for kh = 1:sum(ind_l)
            fill(cost_all_ind(kh) + ([0, cost_lev(ll), cost_lev(ll), 0, 0])/cost_HF,...
                je + 0.5*[-1, -1, 1, 1, -1], color_lev(ll, :), 'EdgeColor', color_lev(ll, :));
        end
        
        if any(ind_l)
            [~, ind_comp] = min(stk_dist(cost_ax'/cost_HF, cost_all(ind_l)'));
            levcost(ind_comp, je) = ll;
        end
    end
end
plot((lim_cost(1) - cost_lev(1)/2)/cost_HF + kron((1:normal_budget)', [1; 1; NaN]),...
    repmat([0; nb_exp + 1; NaN], normal_budget, 1), 'r-', 'LineWidth', 2);
ylim([0; nb_exp + 1]);
xlim([double(lim_cost(1) - cost_lev(1)/2); double(lim_cost(2))]/cost_HF);
set(get(fig, 'CurrentAxes'), 'FontSize', ftz_axi);
stk_xlabel('Cost', 'FontSize', ftz_lab);
stk_ylabel('Experiments', 'FontSize', ftz_lab);
stk_title('Periodogram', 'FontSize', ftz_tit);
save_fig(foldersave, 'ex1_periodogram', fig);

%%
% Histogram of repartition
Y = NaN(normal_budget, nb_lev);
for grp = 1:normal_budget
    idx = (grp - 1)*cost_HF + (1:cost_HF);
    levcost_g = levcost(idx, :);
    
    for ul = 1:nb_lev
        Y(grp, ul) = sum(levcost_g(:) == ul)*cost_lev(ul)/cost_HF;
    end
end

fig = stk_figure('Repartition at each level');
set(fig, 'PaperPositionMode', 'auto');
bar(1:normal_budget, Y);
colormap('parula');
xlim([0; normal_budget + 1]);
set(get(fig, 'CurrentAxes'), 'FontSize', ftz_axi);
stk_xlabel('Part of the experiment', 'FontSize', ftz_lab);
stk_ylabel('Weighted nb. obs', 'FontSize', ftz_lab);
stk_title('Evolution', 'FontSize', ftz_tit);
save_fig(foldersave, 'ex1_weighted_obs', fig);

fig = stk_figure('Proportion at each level');
set(fig, 'PaperPositionMode', 'auto');
bar(1:normal_budget, Y./repmat(sum(Y, 2), 1, nb_lev), 'stacked');
colormap('parula');
xlim([0;normal_budget + 1]);
ylim([0; 1]);
set(get(fig, 'CurrentAxes'), 'FontSize', ftz_axi);
stk_xlabel('Part of the experiment', 'FontSize', ftz_lab);
stk_ylabel('Weighted repartitions', 'FontSize', ftz_lab);
save_fig(foldersave, 'ex1_weighted_prop', fig);
stk_title('Evolution', 'FontSize', ftz_tit);