function res = sec43_oscillator_seq (idx_sim, algorithm, varargin)

save_dir = sec43_oscillator_savedir (idx_sim);

result_prefix = fullfile (save_dir, ...
    sprintf ('sec43_oscillator_%03d_%04d', algorithm.number, idx_sim));

result_file = [result_prefix '.mat'];

if ~ exist (result_file, 'file')
    
    diary ([result_prefix '.log']);
    res = run_experiment_seq_ (idx_sim, algorithm, varargin{:});
    save (result_file, 'res');
    diary ('off');
    
    fprintf ('Simulation #%d, algorithm ''%s'': completed in %.2f hours\n', ...
        idx_sim, algorithm.name, res.run_time / 3600);
    
else  % result already available
    
    tmp = load (result_file);
    res = tmp.res;
    
    fprintf ('Simulation #%d, algorithm ''%s'': data loaded\n', ...
        idx_sim, algorithm.name);
    
end % if

end % function


function res = run_experiment_seq_ (idx_sim, ...
    algorithm, init, problem, data_ref, disp_info)

% SET THIS FLAG TO "true" TO SAVE A LOT OF THINGS
big_save = false;

tic_method = tic ();  start_date = datestr (now);

batch_size = algorithm.batch_size;

% TEMP
cov_opt = init.cov;
lnv_opt = init.lnv;
model = init.model;
param_fixed = init.param_fixed;
is_fixed = init.is_fixed;
bndx = init.bndx;
ns_max = init.ns_max;
frac_prun = init.frac_prun;
nint = init.nint;
do_optim = init.do_optim;
nc = init.nc;
nb_step = init.nb_step;
nb_sim_par = init.nb_sim_par;

% Shorter names: problem
dim = problem.input_dim;
dimx = dim - 1;
budget = problem.budget;

% Prediction grid
[w0_, zeta_] = meshgrid (data_ref.pulse, data_ref.damping);
unit_hcube = init.unit_hcube;
nt = numel (w0_);
xt = stk_dataframe ([w0_(:), zeta_(:), ...
    algorithm.dt_pred * ones(nt, 1)], problem.input_names);
xt_01 = stk_rescale(xt, problem.input_domain, unit_hcube);

xi = init.xi;
zi = init.zi;
design_cost = problem.design_cost(xi);

simul_param = init.simul_param;
SMC_weights = [];

x_new = nan (0, dim);
z_new = nan (0, 1);
time_new = 0;

% Save results
history = stk_dataframe (nan (0, 4), ...
    {'it_num', 'cost', 'prob_estim', 'prob_error', 'pfun_rmse'});
detailed_history = {[]};
time_comp = stk_dataframe (nan (0, 3), ...
    {'smc', 'pred', 'crit'});
if big_save
    simul_param_history = {simul_param};
end

it_num = -1;

while (design_cost + sum(time_new)) <= budget
    
    it_num = it_num + 1;
    
    % While the budget is not exhausted
    if disp_info
        fprintf('Simulation #%d, method ''%s'': cost = %g (%.2f%%)\n', ...
            idx_sim, algorithm.name, design_cost + sum(time_new), ...
            100*(design_cost + sum(time_new))/budget);
        c = clock;
        fprintf('%02d/%02d/%04d at %2dh %02dmin %02ds\n',...
            c(3), c(2), c(1), c(4), c(5), fix(c(6)));
    end
    % Add new points
    xi = cat(1, xi, x_new);
    zi = cat(1, zi, z_new);
    
    design_cost = problem.design_cost (xi);
    
    
    %% Rescale in [0; 1]^3
    
    xi_01 = stk_rescale(xi, problem.input_domain, unit_hcube);
    % Add a line to the time computation
    time_comp = cat(1, time_comp, NaN(1, size(time_comp, 2)));
    
    
    %% Reestimate parameters + Move them according to their new distribution
    
    t1 = tic;
    % Find the Maximum a posteriori
    [cov_opt, lnv_opt] = stk_param_estim(model, xi_01, zi,...
        cov_opt, lnv_opt, @stk_param_mlogpost);
    
    model.param = cov_opt;
    model.lognoisevariance = lnv_opt;
    
    % The vector of hyper-parameters
    vect_estim = stk_get_optimizable_parameters(model);
    param_center = vect_estim(~is_fixed, 1);
    
    % Move the parameters
    if isempty(x_new) % first loop
        SMC_weights = ones(nb_sim_par, 1)/nb_sim_par;
    else
        ni = size(xi_01, 1);
        iniM = 1:(ni - batch_size);
        iniP = (ni - batch_size + 1):ni;
        % SMC - 1: update weights
        SMC_weights = stk_smc_weights(simul_param, model,...
            xi_01(iniM, :), zi(iniM, :), xi_01(iniP, :), zi(iniP, :), SMC_weights);
        % SMC - 2: re-sample
        [simul_param, SMC_weights] = stk_smc_sample(simul_param, SMC_weights);
    end
    
    % SMC - 3: move
    % Select varying parameters
    sim_par = simul_param(:, ~is_fixed);
    % The distribution of parameters
    distrib = @(p)(stk_haario_logpost(model, xi_01, zi, p, param_fixed));
    
    % Box of parameters
    [lb_cov, ub_cov] = stk_param_getdefaultbounds (cov_opt, xi_01, zi);
    [lb_lnv, ub_lnv] = stk_param_getdefaultbounds (lnv_opt, xi_01, zi);
    
    bnd_param = [lb_cov', lb_lnv'; ub_cov', ub_lnv'];
    bnd_param = bnd_param(:, ~is_fixed);
    
    % Covariance for parameters
    cov_param = stk_haario_suggest_C0(distrib, bnd_param, param_center, true);
    
    % Move the parameters
    if disp_info,  disp ('Step 1...');  end
    sim_par = stk_smc_move(sim_par, distrib, nb_step, cov_param);
    % Simulated parameters
    simul_param = repmat(param_fixed', nb_sim_par, 1);
    simul_param(:, ~is_fixed) = sim_par;
    if disp_info,  disp ('Step 1 completed');  end
    time_comp.smc(end) = toc(t1);
    
    if big_save
        simul_param_history{it_num+2} = simul_param;
    end
    
    
    %% Estimation
    
    t2 = tic;
    % Check the MAP parameter
    best_AP = stk_param_mlogpost(model, xi_01, zi);
    best_cov = cov_opt;
    best_lnv = lnv_opt;
    reestimate = false;
    
    % Compute estimation of the probability function
    failp_mean = zeros(nt, 1);
    if disp_info,  disp('Step 2...');  end
    tmp = struct ();  % for big_save
    for kpar = 1:nb_sim_par
        % 1 - change the parameters
        model = stk_set_optimizable_parameters(model, simul_param(kpar, :));
        
        % Bonus - check posterior
        AP = stk_param_mlogpost(model, xi_01, zi);
        if AP <= best_AP
            best_AP = AP;
            best_cov = model.param;
            best_lnv = model.lognoisevariance;
            reestimate = true;
        end
        
        % 2 - compute the prediction, knowing the parameters
        zp_k = stk_predict(model, xi_01, zi, xt_01);
        nv_p_k = stk_covmat_noise (model, xt_01, [], -1, true);
        zp_k_var_obs = zp_k.var + nv_p_k;
        
        % 3 - the local probability of failure, knowing the parameters
        failp_mean_k = stk_distrib_normal_cdf(zp_k.mean,...
            problem.thres, sqrt(zp_k_var_obs));
        
        % 4 - mean over the parameters (reccursive formulation)
        failp_mean = (failp_mean*(kpar - 1) + failp_mean_k)/kpar;
        
        if big_save
            tmp(kpar).zp_mean = zp_k.mean;  %#ok<UNRCH>
            tmp(kpar).zp_var  = zp_k.var;
            tmp(kpar).nv_p    = nv_p_k;
        end
    end

    if big_save
        detailed_history{it_num+2} = tmp;  %#ok<UNRCH>
    end
    
    if disp_info,  disp('Step 2 completed');  end
    
    % Estimate global probability
    prob_estim = mean (failp_mean);
    
    % Compute error
    prob_error = prob_estim - init.prob_glob_ref;    % ^P - P*
    pfun_rmse = sqrt(mean( (failp_mean - init.prob_func_ref).^2 ));
    % ||^p(x) - p*(x)||
    
    % If reestimation is needed, do it
    if reestimate
        if disp_info,  disp ('Reestimation');  end
        [cov_opt, lnv_opt] = stk_param_estim(model, xi_01, zi,...
            best_cov, best_lnv, @stk_param_mlogpost);
    end
    model.param = cov_opt;
    model.lognoisevariance = lnv_opt;
    
    % Save history
    history = [history; ...
        [it_num, design_cost, prob_estim, prob_error, pfun_rmse]]; %#ok<AGROW>
    
    time_comp.pred(end) = toc(t2);
    
    
    %% Design of experiments
    
    t3 = tic;
    
    % Samples randomly the input_X space
    xc_x = stk_sampling_randunif(nc, dimx, bndx);
    
    % Integration point
    xc_int = stk_add_level(xc_x, problem.dt_HF);    % Points at the last level
    xc_int.colnames = problem.input_names;
    xc_int_01 = stk_rescale(xc_int, problem.input_domain, unit_hcube);
    
    % Prune the integration points
    xc_int_01 = stk_sur_pruning (problem.thres, xc_int_01, model, ...
        xi_01, zi, [], frac_prun, nint);
    
    if disp_info,  disp ('Step 3...');  end
    
    % Candidate values fo dt (one or several of them)
    dt_cand_list = [];
    if strcmpi (algorithm.type, 'SUR')
        dt_cand_list = algorithm.dt_SUR;
    elseif strcmpi (algorithm.type, 'MRSUR')
        dt_cand_list = data_ref.step;
    else
        stk_error('Unknown criterion', 'Unkowncase');
    end
    
    % Result at each level (Reduction + best point(s))
    % Compute the reductions and the candidates
    %   * at all levels of fidelity for MR-SUR,
    %   * at a given level for the "fixed-level SUR".
    
    nb_dt_cand = length (dt_cand_list);
    
    x_list = nan (nb_dt_cand, batch_size*dim);
    red_cand = nan (nb_dt_cand, 1);
    for kt = 1:nb_dt_cand
        dt_k = dt_cand_list(kt);
        x_new_k = [];  red_tn = [];
        for kn = 1:batch_size
            [x_cand, red_tn] = SUR_lev (xc_x, dt_k, ...
                problem.input_domain, model, xi_01, zi, ...
                x_new_k, xc_int_01, problem.thres, frac_prun, ...
                ns_max, do_optim);
            % New point
            x_new_k = cat(2, x_new_k, x_cand);
        end
        x_list(kt, :) = x_new_k;
        red_cand(kt, 1) = red_tn;
    end
    
    % Compute the time for each candidate
    cost_cand = sum (problem.CPUTime ...
        (x_list(:, dim:dim:(batch_size * dim))), 2);
    cost_cand(cost_cand + design_cost > budget) = +Inf;
    % Look for the best level
    rate = red_cand ./ cost_cand;
    [~, selected_level] = max (rate);
    
    % New (set of) point(s)
    x_new = x_list(selected_level, :);
    
    % A reshape is needed now if batch_size > 1
    x_new = (reshape (x_new, dim, batch_size))';
    
    % Launch the simulator at the new input point(s),
    % record the output and the coresponding (known) computation time
    z_new = nan (batch_size, 1);
    time_new = nan (batch_size, 1);
    for idx_obs = 1:batch_size
        x_new_ = x_new(idx_obs, :);
        % Launch the simulator at this new value
        z_new(idx_obs, 1) = damped_harmonic_simulator(x_new_(1, 1), x_new_(1, 2), x_new_(1, 3));
        % Save the required time
        time_new(idx_obs) = problem.CPUTime (x_new_(1, 3));
    end
    
    if disp_info,  disp ('Step 3 completed'); disp(' ');  end
    
    time_comp.crit(end) = toc(t3);
end

% Info struct
info = get_config_info ();
info.start_date = start_date;
info.end_date = datestr (now);

% Save results
res = sec43_oscillator_create_res ();

res.algorithm        = algorithm;
res.idx_sim          = idx_sim;
res.batch_size       = batch_size;
res.history          = history;
res.detailed_history = detailed_history;
res.time_comp        = time_comp;
res.xi               = xi;
res.zi               = zi;
res.ni_init          = init.ni;
res.dt               = data_ref.step;
res.model            = model;

if ~ big_save
    res.simul_param = simul_param;  % save the last one only
else
    res.simul_param = simul_param_history;
end

res.info        = info;
res.run_time    = toc (tic_method);

end % function
