function [X, t, V] = linear_oscillator(w0, zeta, dt, t_end, S, X0, V0, method)
% LINEAR_OSCILLATOR solves by finite difference the second degree
% oscillator.
%
% CALL : [X, t, V] = linear_oscillator(w0, zeta, dt, t_end, S, X0, V0, method)
%
%   solves the equation d(X) = dt*(    0          1)(X) + dBt
%                        (V)      (-w0^2 -2*zeta*w0)(V)
%
%   with :
%       - dt : the time step (in s);
%       - w0 : the pulsation (in rad/s);
%       - zeta : the damping ration (without unit);
%       - t_end : the end of the simulation (in s);
%       - S : the spectral intensity of the white noise,
%             dBt ~ N(0, 2*pi*S*dt);
%       - X0, V0 : the initial conditions;
%       - method : a string indicating which method to use for solving
%                  equations. Can be 'explicit', 'implicit' or
%                  'exponential' (default);
%       * X : the position;
%       * t : the time;
%       * V : the speed.
%
% REFERENCE
%
%   AU, Siu-Kui and BECK, James L. Estimation of small failure
%   probabilities in high dimensions by subset simulation.
%   Probabilistic Engineering Mechanics, 2001, vol. 16, no 4, p. 263-277.
%
% See also : damped_harmonic_simulator

if nargin < 2
    error('Not enough input arguments');
end

if nargin > 8
    error('Too many input arguments');
end

%% Complete default arguments
% Time step (in second)
if nargin < 3 || isempty(dt)
    dt = min(zeta, 1/(zeta + sqrt(zeta^2 - 1)))/w0;
end

% Time horizon (in second)
if nargin < 4 || isempty(t_end)
    t_end = 30;
end

% Spectral intensity of the white noise
if nargin < 5 || isempty(S)
    S = 1;
end

% Initial position
if nargin < 6 || isempty(X0)
    X0 = 0;
end

% Initial speed
if nargin < 7 || isempty(V0)
    V0 = 0;
end

% Method
if nargin < 8 || isempty(method)
    method = 'exponential';
else
    method = lower(method);
end

% Check positivity
if any([w0, zeta, dt, t_end, S] < 0)
    error('w0, zeta, dt, t_end and S must be positive.');
end

% If method is Euler explicit ==> check the value of dt
if strcmpi(method, 'explicit')
    dt0 = 2/w0;
    dtLim = dt0*min(zeta, 1/(zeta + sqrt(zeta^2 - 1)));
    if dt >= dtLim
        warning(['Risk of instabilities : dt is too large.\n', ...
            'Recommanded maximal value : %g'], dtLim);
    end
end

%% Axis of time
t = 0:dt:t_end;             % Time
if t(end) < t_end
    t = [t, t(end) + dt];
end
nt = length(t);             % Number of step

%% Sample the equation
Y = NaN(2, nt);
Y(:, 1) = [X0; V0];             % Initial conditions

sigma = realsqrt(2*pi*S*dt);    % Noise standard deviation

A = [0, 1;
    -w0^2, -2*zeta*w0;];        % Linear drift
A = A*dt;                       % Add dt effect

switch method
    case 'explicit'
        Y = explicit(Y, nt, A, sigma);      % Stable if dt < dtLim
    case 'implicit'
        Y = implicit(Y, nt, A, sigma);      % Always stable
    case 'exponential'
        Y = exponential(Y, nt, A, sigma);   % Always stable
    otherwise
        warning('Unknown solving method. Return NaN.');
        Y(:, 1) = NaN(2, 1);
end

%% Return results
X = Y(1, :);
V = Y(2, :);

end

%% Methods
function Y = explicit(Y, nt, A, sigma)
% explicit Euler scheme

L = eye(2) + A;

for k = 2:nt
    Y(:, k) = L*Y(:, k - 1) + [0; sigma*randn(1, 1)];
end

end

function Y = implicit(Y, nt, A, sigma)
% implicit Euler scheme

K = eye(2) - A;
d = K(1, 1)*K(2, 2) - K(2, 1)*K(1, 2);

L = [K(2, 2), -K(1, 2); -K(2, 1), K(1, 1)]/d;
% Inverse of [eye(2) - A]

for k = 2:nt
    Y(:, k) = L*(Y(:, k - 1) + [0; sigma*randn(1, 1)]);
end

end

function Y = exponential(Y, nt, A, sigma)
% explicit exponential Euler scheme (Lord & Rougemont)

L = expm(A);

for k = 2:nt
    Y(:, k) = L*( Y(:, k - 1) + [0; sigma*randn(1, 1)] );
end

end