% STK_SMC_MOVE moves the particles
%
% CALL: MOVED_PARAM = stk_smc_move(SAMP_PARAM, LOGPOSTERIOR)
%
%   moves the sampled parameters using a Metropolis-Hastings algorithm of
%   each individual parameters. SAMP_PARAM are the current parameters,
%   LOGPOSTERIOR must be the +logarithm of the posterior distribution of
%   hyper-parameters.
%   The proposal function is a normal distribution.
%
% CALL: MOVED_PARAM = stk_smc_move(SAMP_PARAM, LOGPOSTERIOR, T0)
%
%   does the same thing, but changes the length of the Metropolis-Hastings
%   chains.
%   Default value: T0 = 20.
%
% CALL: MOVED_PARAM = stk_smc_move(SAMP_PARAM, LOGPOSTERIOR, T0, COV_MCMC)
%
%   does the same thing, but changes the covariance of the proposal
%   distribution by COV_MCMC.
%   Default value: COV_MCMC = ((2.4^2)/size(SAMP_PARAM, 2))*cov(SAMP_PARAM).
%
% CALL: [MOVED_PARAM, ACCEPT_RATIO] = stk_smc_move(SAMP_PARAM, LOGPOSTERIOR, T0, COV_MCMC)
%
%   returns also the acceptance ratio of the Metropolis-Hastings algorithm.
%   ACCEPT_RATIO is a size(SAMP_PARAM, 1) x 1 vector, one value by
%   sampling.
%
% See also: stk_smc_sample, stk_smc_weights, stk_haario_adaptive_metropolis
function [moved_param, accept_ratio] = stk_smc_move(samp_param, logpost, t0, cov_MCMC)

if nargin > 4
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

if nargin < 2
    stk_error('Not enough input arguments', 'NotEnoughInputArgs');
end

%% Inputs
nb_param = size(samp_param, 1);
dim_param = size(samp_param, 2);

if nargin < 3 || isempty(t0)
    t0 = 20;
else
    if ~isnumeric(t0)
        try
            t0 = double(t0);
        catch
            stk_error ('t0 must be a number.', 'InvalidArgument');
        end
    end
    if numel(t0) > 1
        warning('STK:stk_smc_move:oneValueForT0',...
            'There need be only one value for t0. t0(1) was chosen.');
        t0 = t0(1); % only one value
    end
end

sd = ((2.4)^2)/dim_param;   % Following the advice of Haario

if nargin < 4 || isempty(cov_MCMC)
    cov_MCMC = sd*cov(samp_param);
    eig_cov = eig(cov_MCMC);
    
    % Check that covariance is OK
    if (min(eig_cov) <= 0) || (min(abs(eig_cov))/max(abs(eig_cov)) < sqrt(eps))
        % if the covariance is bad conditioned
        cov_MCMC = diag(diag(cov_MCMC)); % use the diagonal of variance matrix
    end
    
else
    if ~isequal(size(cov_MCMC), [dim_param, dim_param])
        stk_error (['cov_MCMC must be a square matrix,',...
            ' whose size is equal to the dimension of the parameters.'], 'IncorrectSize');
    end
    
    % Symmetry
    if ~isequal(cov_MCMC, cov_MCMC')
        stk_error (['cov_MCMC must be a positive symmetric square matrix,',...
            ' but your matrix is not symmetric.'], 'InvalidArgument');
    end
    % Positivity
    eig_cov = eig(cov_MCMC);
    if min(eig_cov) <= 0
        stk_error (['cov_MCMC must be a positive symmetric square matrix,',...
            ' but your matrix is not strictly positive.'], 'InvalidArgument');
    end
    
    if (min(abs(eig_cov))/max(abs(eig_cov)) < sqrt(eps))
        stk_error ('Your matrix cov_MCMC is badly-conditioned.', 'InvalidArgument');
    end
end

if max(diag(cov_MCMC)) < sqrt(eps)
    warning('STK:stk_smc_move:matrixNearToZero',...
        'The larger diagonal element of the matrix ''cov_MCMC'' is near to zero.');
end

%% Other used data

moved_param = samp_param;

isLog = true;           % is the log-posterior
epsilon = [];               % use default value for epsilon

% Large bounds of the parameters
bnd_param = [min(samp_param);
    max(samp_param);] + 3*[-1; 1]*sqrt(diag(cov_MCMC))';

algo_MCMC = 'hastings';     % No adaptation
t1 = 0;
t2 = 0;

display_waitbar = false;    % Do not display a waiting bar during each Metropolis

%% Move and scatter hyper-parameters
accept_ratio = NaN(nb_param, 1);
for kpar = 1:nb_param
    [moved_param(kpar, :), accept_ratio(kpar)] = stk_haario_adaptive_metropolis(logpost, bnd_param, 1,...
        'x0', moved_param(kpar, :), 'C0', cov_MCMC,...
        't0', t0, 't1', t1, 't2', t2, 'logDistrib', isLog, 'epsilon', epsilon,...
        'updating', algo_MCMC, 'display', display_waitbar);
    
end

end

