% STK_HAARIO_ADAPTIVE_METROPOLIS samples parameters according to a distribution
%
% CALL: [XF, ACCEPT] = stk_haario_adaptive_metropolis(DISTRIB, BOX_PARAM, NF)
%
%   uses the Haario algorithm to generate NF samples of the probability
%   distribution DISTRIB.
%   * DISTRIB is a function, which takes 1 x dim matrix, and which returns
%       a positive value.
%   * BOX_PARAM is a 2 x dim matrix, which contains the bounds of the
%       domain of DISTRIB. BOX_PARAM is used to calibrate the initial
%       parameters of the function.
%   The algorithm is like a standard Metropolis-Hasting algorithm, with a
%   Gaussian distribution as proposal, but the covariance of the proposal
%   is adapted as function of generate parameters.
%   XF is a NF x dim matrix, such as XF(k, :) are samples of DISTRIB.
%   ACCEPT is the acceptation rate of the algorithm.
%
% CALL: [XF, ACCEPT] = stk_haario_adaptive_metropolis(DISTRIB, BOX_PARAM, NF,...
%          X0, C0, T0, T1, T2, LOGDISTRIB, EPSILON, UPDATING, DISPLAY)
%
%   changes the default values of the algorithm. They can be let empty (in
%   this case, default value will be used).
%   - X0: the initial point of the Metropolis algorithm.
%       Default value: X0 = (BOX_PARAM(1, :) + BOX_PARAM(2, :))/2
%   - C0: the initial covariance of the proposal.
%       Default value:
%           C0 = (sqrt(sd)*diag((BOX_PARAM(2, :) - BOX_PARAM(1, :))/25))^2
%       with sd = 2.4^2/dim
%   The initial proposal function follows a N(X0, C0) distribution.
%   - T0, T1, T2: three integers which rules the steps of the algorithm.
%       T0 defines the burn-in, when the Metropolis algorithm is not
%       adaptive (Cn = C0).
%       T1 is the number of iterations between the end of burn-in, and
%       the first point returned by the algorithm.
%       T2 is the number of iterations between 2 successive points of XF.
%       The total number of iterations is tf = T0 + (T1 - T2) + NF*T2.
%       Default values: T0 = 1000; T1 = 5000; T2 = 50.
%   - LOGDISTRIB: a boolean indicating if DISTRIB is the probability
%       distribution, or its logarithm. Can improve the accuracy.
%       Warning: if LOGDISTRIB is true, DISTRIB must be the logarithm,
%       not its opposite.
%       Default value: false (DISTRIB is the real distribution).
%   - EPSILON: a small value compared to C0. Use to avoid numerical
%       instabilities.
%   - UPDATING: a char list, which indicates which updating formula to
%       use. Allowed values are 'hard', 'smooth' or 'hastings'.
%       Default value: 'hard' (proposed by Haario).
%       If UPDATING = 'hastings', then there is not any adaptation,
%       the algorithm is a standard Metropolis-Hastings algorithm with a
%       N(., C0) distribution as proposal.
%   - DISPLAY: a boolean indicating if a progress bar must be displayed or not.
%       Default value: true.
%
% CALL: [XF, ACCEPT] = stk_haario_adaptive_metropolis(DISTRIB, BOX_PARAM, NF,...
%                           'Property1', Value1, 'Property2', Value2, ...)
%
%   is an other method to change the options.
%
% REFERENCE
%
%   [1] Heikki Haario, Eero Saksman, and Johanna Tamminen, "An adaptive
%       metropolis algorithm", Bernoulli, 7(2):223-242, 2001.
%
% See also: stk_haario_suggest_C0, stk_haario_autocorr, stk_param_relik
function [xf, accept, chains] = stk_haario_adaptive_metropolis(distrib, box, nf, varargin)


%% Check arguments
% check required arguments
if nargin < 2 || isempty(distrib) || isempty(box)
    stk_error ('You must define the distribution and the parameter box.',...
        'NotEnoughInputArgs');
end

% test box
if ~ isa (box, 'stk_hrect')
    box = stk_hrect (box);%check size(box) = 2 x dim and box(1, :) <= box(2, :)
end
box = double(box);  %to apply operations on box (division)
dim = size(box, 2);

% Number of points in output
if nargin < 3 || isempty(nf)
    nf = 100;	% Number of final points randomly drawn
end
xf = NaN(nf, dim);    % Final result. Check is nf is correct.


% Other data
[x0, sd, C0_12, C0, t0, t1, t2, logDistrib, epsilon, updating, display_waitbar]...
    = parseInputArgs(box, varargin);

tf = t0 + t1 + (nf - 1)*t2; % number of iterations
chains = NaN(tf + 1, dim);

%% Adaptive Metropolis
xn = x0;
chains(1, :) = xn;
kf = 1;
distrib_n = distrib(xn);
accept = 0;
Cn_12 = C0_12;                  % initial values

Kn = zeros(dim, dim);           % Kn = cov(x0, x1, x2, ..., xn)
Mn = xn;                        % Mn =mean(x0, x1, x2, ..., xn)

if logDistrib
    if numel(distrib_n) > 1  % Check the log-distribution
        stk_error('The log-distribution function must return one value.',...
            'InvalidArgument');
    end
else
    if numel(distrib_n) > 1 || distrib_n <= 0   %Check the distribution
        stk_error('The distribution must return one positive value.',...
            'InvalidArgument');
    end
end

if display_waitbar
    name_waitbar = 'Haario algorithm';
    mes_wait = 'Sampling the distribution:';
    % Display progress
    advan = waitbar(0, sprintf('%s %.1f%%.', mes_wait, 0), 'name', name_waitbar);
end

for nt = 1:tf
    yt = xn + randn(1, dim)*Cn_12;	% Draw a random points from a N(xn, Cn)
    distrib_y = distrib(yt);        % Compute the distribution at this point
    
    if logDistrib
        alpha = min(0, distrib_y - distrib_n);
        unif = log(rand(1));    % -exprnd
        % u ~ U([0; 1]) <=> -log(u) ~ Exp(1)
        % P( u <= exp(a - b) ) = P( log(u) <= (a - b) )
    else
        if distrib_y < 0    %Check the distribution
            stk_error('The distribution function must return a positive value.',...
                'InvalidArgument');
        end
        alpha = min(1, distrib_y/distrib_n);
        unif = rand(1);
    end
    
    % If success, change the point
    if unif <= alpha
        xn = yt;
        distrib_n = distrib_y;
        accept = accept + 1;
    end     % else, keep the old point
    
    [Cn_12, Mn, Kn] = updateCovariance(xn, Cn_12, Mn, Kn, nt, sd, C0, t0,...
        epsilon, updating);
    % Adaptive algorithm
    
    chains(nt + 1, :) = xn;
    if nt >= (t0 + t1 + (kf - 1)*t2)    % Save the simulated parameters
        % after the burn-in t0 + t1, and at each t2.
        xf(kf, :) = xn;
        kf = kf + 1;
    end
    
    
    if display_waitbar
        % Update the waiting bar
        if ishandle(advan)
            % Prevent from the case where the user closed the waiting bar.
            waitbar(nt/tf, advan, sprintf('%s %.1f%%', mes_wait, 100*nt/tf));
        else
            advan = waitbar(nt/tf, sprintf('%s %.1f%%', mes_wait, 100*nt/tf),...
                'name', name_waitbar);
        end
    end
end

if display_waitbar && ishandle(advan)
    close(advan);   % close the waiting bar
end
accept = accept/tf;

end


%% Parse inputs
function [x0, sd, C0_12, C0, t0, t1, t2, logDistrib, epsilon, updating, display_waitbar]...
    = parseInputArgs(box, varargin)

dim = size(box, 2);
%% Default values
x0 = (box(1, :) + box(2, :))/2; % default value: center of the domain
sd = ((2.4)^2)/dim;             % Following the advice of Haario

% C0_12 = sqrt(sd)*diag( (box(2, :) - box(1, :))/25 );
C0 = sd*diag( ((box(2, :) - box(1, :)).^2)/625 );
% Default value for C0_12: 1/5 of the size of box along each dimension

t0 = 1000;                  % Burn out
t1 = 5000;                  % Burn out
t2 = 50;                    % Interval between 2 simulated points

logDistrib = false;         % By default, the function is the real distribution (not its logarithm)

defaultEpsilonValue = true; % By default, use the default value...

updating = 'hard';          % Default updating formula: formula (3) in Haario (2001)

display_waitbar = true;     % Default display a wait bar
%% Change arguments
argsInput = varargin{:};
lenVar = length(argsInput);
if ((lenVar < 8) && all(~cellfun(@ischar, argsInput))) ||...
        ((lenVar >= 8) && all(~cellfun(@ischar, argsInput(1:7))))
    %% First parsing function
    %Suppose the arguments are imputed in this order:
    % {distrib, box, nf, x0, C0, t0, t1, t2, logDistrib, epsilon, updating, display_waitbar}
    if lenVar > 9
        stk_error ('Too many input arguments.', 'TooManyInputArgs');
    end
    
    % x0
    if lenVar >= 1 && ~isempty(argsInput{1})
        x0 = argsInput{1};
    end
    
    % C0
    if lenVar >= 2 && ~isempty(argsInput{2})
        C0 = argsInput{2};
    end
    
    % t0
    if lenVar >= 3 && ~isempty(argsInput{3})
        t0 = argsInput{3};
    end
    
    % t1
    if lenVar >= 4 && ~isempty(argsInput{4})
        t1 = argsInput{4};
    end
    
    % t2
    if lenVar >= 5 && ~isempty(argsInput{5})
        t2 = argsInput{5};
    end
    
    % logDistrib
    if lenVar >= 6 && ~isempty(argsInput{6})
        logDistrib = argsInput{6};
    end
    
    % epsilon
    if lenVar >= 7 && ~isempty(argsInput{7})
        epsilon = argsInput{7};
        defaultEpsilonValue = false;
    end
    
    % updating
    if lenVar >= 8 && ~isempty(argsInput{8})
        updating = argsInput{8};
    end
    
    % display_waitbar
    if lenVar >= 9 && ~isempty(argsInput{9})
        display_waitbar = argsInput{9};
    end
else
    %% Second parsing function
    %Suppose the arguments are imputed in this order:
    % {distrib, box, nf, 'nameArg1', valueArg1, 'nameArg2', valueArg2, ...}
    klV = 1;
    while klV < lenVar
        if ischar(argsInput{klV})
            % Find the associated property
            if     strcmpi(argsInput{klV}, 'x0')
                x0 = argsInput{klV + 1};
                
            elseif strcmpi(argsInput{klV}, 'C0')
                C0 = argsInput{klV + 1};
                
            elseif strcmpi(argsInput{klV}, 't0')
                t0 = argsInput{klV + 1};
                
            elseif strcmpi(argsInput{klV}, 't1')
                t1 = argsInput{klV + 1};
                
            elseif strcmpi(argsInput{klV}, 't2')
                t2 = argsInput{klV + 1};
                
            elseif strcmpi(argsInput{klV}, 'logDistrib')
                logDistrib = argsInput{klV + 1};
                
            elseif strcmpi(argsInput{klV}, 'epsilon')
                epsilon = argsInput{klV + 1};
                defaultEpsilonValue = isempty(epsilon);
                
            elseif strcmpi(argsInput{klV}, 'updating')
                updating = argsInput{klV + 1};
                
            elseif strcmpi(argsInput{klV}, 'display')
                display_waitbar = argsInput{klV + 1};
                
            else %warning
                warning('STK:stk_haario_adaptive_metropolis:unknownArgument',...
                    ['The option ', argsInput{klV}, ' does not exist.']);
            end
            
            klV = klV + 1;
        end %if ischar(varargin{klV})
        
        klV = klV + 1;
    end %while klV < lenVar
end

%% Check values
%x0
if ~ isequal (size(x0), [1, dim])
    %check size(x0) = 1 x dim
    if isequal(size(x0), [dim, 1])
        x0 = x0';
    else
        stk_error (['x0 should be a row,',...
            ' with the same number of columns as box.'], 'IncorrectSize');
    end
end
if any( (x0(1, :) < box(1, :)) + (x0(1, :) > box(2, :)) )
    %check that every element of x0 is higher than box(1, :) and lower
    %than box(2, :).
    stk_error ('x0 must be include inside the box.', 'InvalidArgument');
end

%C0
if ~isequal(size(C0), [dim, dim])
    stk_error (['C0 must be a positive symmetric square matrix,',...
        ' whose size is equal to size(box, 2).'], 'IncorrectSize');
end
C0_12 = chol(C0);   % check symmetry and positivity

%t0
if ~isnumeric(t0)
    try
        t0 = double(t0);
    catch
        stk_error ('t0 must be a number.', 'InvalidArgument');
    end
end
if numel(t0) > 1
    warning('STK:stk_haario_algorithm:oneValueForT0',...
        'There need be only one value for t0. t0(1) was chosen.');
    t0 = t0(1); % only one value
end

%t1
if ~isnumeric(t1)
    try
        t1 = double(t1);
    catch
        stk_error('t1 must be a number.', 'InvalidArgument');
    end
end
if numel(t1) > 1
    warning('STK:stk_haario_algorithm:oneValueForT1',...
        'There need be only one value for t1. t1(1) was chosen.');
    t1 = t1(1); % only one value
end

%t2
if ~isnumeric(t2)
    try
        t2 = double(t2);
    catch
        stk_error('t2 must be a number.', 'InvalidArgument');
    end
end
if numel(t2) > 1
    warning('STK:stk_haario_algorithm:oneValueForT2',...
        'There need be only one value for t2. t2(1) was chosen.');
    t2 = t2(1); % only one value
end

%logDistrib
if ~islogical(logDistrib)
    try
        logDistrib = logical(logDistrib);
    catch
        stk_error('logDistrib must be a logical.', 'InvalidArgument');
    end
end
if numel(logDistrib) > 1
    warning('STK:stk_haario_algorithm:oneValueForLogDistrib',...
        'There need be only one value for logDistrib. logDistrib(1) was chosen.');
    logDistrib = logDistrib(1); % only one value
end

%epsilon
if defaultEpsilonValue
    epsilon = min([
        prod(box(2, :) - box(1, :))/1000;       % epsilon is small in front of the volume of box
        min( (box(2, :) - box(1, :)).^2/25000); % epsilon is small in front of each size of box
        min(eig(C0))/1000                       % epsilon is small in front of C0
        ]);
else
    if ~isnumeric(epsilon)
        try
            epsilon = double(epsilon);
        catch
            stk_error('epsilon must be a numeric.', 'InvalidArgument');
        end
    end
end

if isscalar(epsilon)    % case numel(epsilon) == 1 ==> return diag(epsilon)
    epsilon = epsilon*eye(dim);
elseif isvector(epsilon) % case numel(epsilon) == dim
    if length(epsilon) ~= dim
        stk_error ('If epsilon is a vector, its size must be equal to size(box, 2)',...
            'IncorrectSize');
    end
    epsilon = diag(epsilon);
else % if ismatrix(epsilon)
    if ~isequal(size(epsilon), [dim, dim])
        stk_error (['If epsilon is a matrix, it must be a square matrix,',...
            ' whose size is equal to size(box, 2)'], 'IncorrectSize');
    end
end

%updating
if iscellstr(updating)
    updating = updating{1};
end

if ~ischar(updating) ||...
        (~strcmpi(updating, 'hard') && ~strcmpi(updating, 'smooth') && ~strcmpi(updating, 'hastings'))
    stk_error('updating must be a ''hard'', ''smooth'' or ''hastings''.', 'InvalidArgument');
end
updating = lower(updating); % assert lower case

%display_waitbar
if ~islogical(display_waitbar)
    try
        display_waitbar = logical(display_waitbar);
    catch
        stk_error('display_waitbar must be a logical.', 'InvalidArgument');
    end
end
if numel(display_waitbar) > 1
    warning('STK:stk_haario_algorithm:oneValueForWaitBar',...
        'There need be only one value for display_waitbar. display_waitbar(1) was chosen.');
    display_waitbar = display_waitbar(1); % only one value
end
end

%% Updating covariance
function [Cn_12, Mn, Kn] = updateCovariance(xn, CnM1_12, MnM1, KnM1, nt, sd, C0,...
    t0, epsilon, updating)



if strcmpi(updating, 'hastings')
    % Use default values for future mean and covariance
    Mn = MnM1;
    Kn = KnM1;
    
    % If standard Metropolis-Hastings algorithm, no adaptation
    Cn_12 = CnM1_12;
    % Allow to avoid a Cholevsky decomposition
else
    Mn = (nt*MnM1 + xn)/(nt + 1);   %update mean (M{n + 1})
    Kn = ( (nt - 1)*KnM1 + nt*(MnM1'*MnM1) - (nt + 1)*(Mn'*Mn) + (xn'*xn) )/nt;
    %update Covariance (formula (3) in Haario (2001), with epsilon=0, and sd=1)
    
    switch updating
        case 'hard' % hard formula (hard change at t = t0 + 1)
            if nt <= t0
                Cn = C0;
            else
                Cn = sd*(Kn + epsilon);
            end
        case 'smooth'   % smooth formula
            Cn = (t0*C0 + (nt + 1)*sd*(Kn + epsilon))/(t0 + nt + 1);
        otherwise
            stk_error(['There is only three updating formulas: ',...
                '''hard'', ''smooth'', and ''hastings''.'], 'InvalidArgument');
    end
    
    % Cholevsky decomposition.
    Cn_12 = chol(Cn);
end

end