%STK_HAARIO_AUTOCORR computes the auto-correlation of a matrix
%
% CALL: CORR_MAT = stk_haario_autocorr(XF, NB)
%
%   computes the auto-correlation chains of each columns of XF, for 0 to NB.
%   More exactly, if size(XF) = nf x dim, then corr_mat is a (NB + 1) x dim
%   matrix such as corr_mat(i, j) is the auto-correlation of XF(:, j) at i
%   - 1. corr_mat(1, j) = 1.
%   This function can be used to check the correlation of the parameter
%   chains returned by stk_haario_adaptive_metropolis.
%
% CALL: CORR_MAT = stk_haario_autocorr(XF)
%
%   do the same thing, but with the default value
%   NB = min( floor((nf - 1)/2), 30)
%
% See also: stk_haario_adaptive_metropolis
function corr_mat = stk_haario_autocorr(xf, nb)

if nargin > 2
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

if nargin < 1
    stk_error('Not enough input arguments', 'NotEnoughInputArgs');
end

%% Check inputs

dim_param = size(xf, 2);    % dimension of vector of parameters
nf = size(xf, 1);           % number of different parameters

if nf < 2
    stk_error('You cannot compute an auto-correlation with one value.', 'OneRowOnly');
end

% Delay between auto-correlation
if nargin < 2 || isempty(nb)
    nb = min( floor((nf - 1)/2), 30);
end

if nb < 0
    stk_error('The larger of auto-correlation evaluation must be greater than 0.', 'PositiveArgRequire');
end
nb = floor(nb); % assert integer

if nf < (nb + 2)
    stk_error(['The larger of auto-correlation evaluation must be lower',...
        ' than the number of simulate parameters.'], 'NotEnoughRow');
end

%% Compute correlation
corr_mat = NaN(nb + 1, dim_param);
var_xf = var(xf, 0, 1); % 0: divide by nf - 1; 1: along the first dimension

% if variance equals to 0 => no variation => correlation equals to 1
is_var0 = (var_xf == 0);
corr_mat(:, is_var0) = 1;

% Find index with variation
xd = xf(:, ~is_var0);                   % Select varying lists
xd = xd -  repmat(mean(xd, 1), nf, 1);  % center the values

for tb = 0:nb
    % compute the auto-covariance
    idx_m = 1:(nf - tb);
    idx_p = (1 + tb):nf;
    cov_tb = sum(xd(idx_m, :).*xd(idx_p, :), 1)/(nf - tb - 1);
    % Covariance
    
    corr_mat(tb + 1, ~is_var0) = cov_tb./var_xf(~is_var0);
    % divide by the term in 0 ==> correlation
end


end

