% STK_HAARIO_CORR_INDE computes the quantile of the empirical random correlation
%
% CALL: EMP_COR = stk_haario_corr_inde(N, P)
%
%   computes the empirical correlation between two Gaussian random vectors,
%   independent and identically distributed according to a N(0, eye(N))
%   distribution. The function samples ceil(1e6/N) independent vectors,
%   computes the empirical correlation, then returns the empirical
%   quantiles EMP_COR of the correlation at P%.
%   P must be between 0 and 1.
%
%   Can be used to see if the auto-correlation in a chain is significant
%   or not.
%
% CALL: [EMP_COR, NTEST] = stk_haario_corr_inde(N, P)
%
%   returns also NTEST, the number of samples used to compute the empirical
%   correlation.
%   NTEST = ceil (1e6/N);
%
% See also: stk_haario_autocorr, stk_haario_figure, corrcoef
function [emp_cor, ntest] = stk_haario_corr_inde(n, p)

if nargin > 2
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

if nargin < 1
    stk_error('Not enough input arguments', 'NotEnoughInputArgs');
end

%% Check inputs arguments
n = ceil(abs(n(1)));    % Assert n is a scalar strictly positive integer.

if nargin < 2 || isempty(p)
    p = (1 + 0.95*[-1; 1])/2;
else
    if any(p(:) < 0)
        stk_error('Probabilities must be positive.', 'InvalidArgument');
    elseif any(p(:) > 1)
        stk_error('Probabilities must be lower than 1.', 'InvalidArgument');
    end
end

ntest = ceil(1e6/n);	% Number of tests (Maximum according to my personal memory)
rho = 0.0;              % Real value of the correlation
% TO DO: add ntest and rho in the input arguments

%% Simulate empirical correlations
% Sigma = [1, rho;
%     rho, 1];    % covariance matrix
CholS = [1, rho;
    0, sqrt(1 - rho^2)];    % Cholevsky decomposition
%assert(norm(Sigma - CholS'*CholS, 'fro') < sqrt(eps));

% Random generation
X = randn(n, ntest);  % first data
Y = randn(n, ntest);  % second data

% Add correlation
Y = CholS(1, 2)*X + CholS(2, 2)*Y;

% Centering
Xc = X - repmat(mean(X), n, 1);
Yc = Y - repmat(mean(Y), n, 1);

% Standard deviation
Sx = std(X, 1, 1);
Sy = std(Y, 1, 1);
% The empirical correlation requires to compute the biased standard
% deviation, and so, to divide by n (instead of n - 1)

% Calculation of empirical correlation
covar = mean(Xc.*Yc);
corr = covar./(Sx.*Sy);

%% Return the empirical correlation
% If rho == 0, quantile at p% + quantile at (1 - p)% = 0
if rho == 0
    emp_cor1 = quantile(corr, p);
    emp_cor2 = quantile(corr, 1 - p);
    emp_cor = (emp_cor1 - emp_cor2)/2;
else
    emp_cor = quantile(corr, p);
end

end