% STK_HAARIO_FIGURE draws figure to represent the output of a sampling
%
% CALL: H = stk_haario_figure(XF)
% CALL: H = stk_haario_figure(XF, 'corr')
%
%   plots the correlation of the chains XF(:, k) on an unique figure. H is
%   the handle of the figure.
%
% CALL: H = stk_haario_figure(XF, 'param')
%
%   plots the values of XF(:, k) as function of their row numbers. There
%   are one sub-plot figures by column of XF. H is the handle of the
%   figure.
%
% CALL: H = stk_haario_figure(XF, 'corr_param')
%
%   plots the correlation of the chains XF(:, k), with one sub-plot figures
%   by column of XF. H is the handle of the figure.
%
% CALL: H = stk_haario_figure(XF, 'param', MODEL)
% CALL: H = stk_haario_figure(XF, 'corr_param', MODEL)
%
%   use a sub-routine to find the name of the parameters from the STK MODEL.
%
% EXAMPLE
%
%   dim = 2;
%   model = struct('covariance_type', @stk_materncov52_aniso, 'dim', dim);
%   xf = randn(1000, dim + 1);	% Pseudo-sampling of the hyper-parameters
%
%   h1 = stk_haario_figure(xf, 'corr', model);
%   h2 = stk_haario_figure(xf, 'param', model);
%   h3 = stk_haario_figure(xf, 'corr_param', model);
%
% See also: stk_haario_adaptive_metropolis, stk_haario_autocorr
function handle = stk_haario_figure(xf, typeOf, model)

if nargin > 3
    stk_error('Too many input arguments', 'TooManyInputArgs');
elseif nargin < 1
    stk_error('Not enough input arguments', 'NotEnoughInputArgs');
end

%% Parameters of the figure
% Is handle?
get_han = (nargout > 0);

% Name of the hyper-parameters
if nargin < 2 || isempty(typeOf)
    typeOf = 'corr';
end

%% Find the name of the parameters
if ~isempty(strfind(typeOf, 'param'))
    dim_param = size(xf, 2);
    if nargin < 3 || isempty(model)
        param_name = stk_find_name(dim_param);
    else
        param_name = stk_find_name(dim_param, model);
    end
end     % otherwise, do not search, it is useless

%% Compute the correlation
if ~isempty(strfind(typeOf, 'corr'))
    corr_mat = stk_haario_autocorr(xf);
    
    nb_param  = size(xf, 1);
    % Find the significant value to say "the chains are not independent".
    prob_signi = 0.95;  % Confidence at 95%
    significant_cor = stk_haario_corr_inde(nb_param, (1 + prob_signi*[-1; 1])/2);
end     % otherwise, do not compute the correlation

%% Plot figures
if strcmpi(typeOf, 'corr')
    hand = stk_haario_figure_auto(corr_mat, significant_cor);
elseif strcmpi(typeOf, 'param')
    hand = stk_haario_figure_param(xf, param_name);
elseif strcmpi(typeOf, 'corr_param')
    hand = stk_haario_figure_autocorr(corr_mat, param_name, significant_cor);
else
    stk_error('Unknown type of figure.', 'UnknownTypeFig');
end

%% Return output
if get_han
    handle = hand;
end

end

%% Function which plot data
function hand = stk_haario_figure_auto(corr_mat, significant_cor)
% Plot correlation on an unique graphic
nb_corr = size(corr_mat, 1) - 1;
%% Figure
hand = stk_figure('Auto-correlations');
hold on;
% zeros axis
plot([0; nb_corr], zeros(2, 1), 'k-');
% Correlations
plot(0:nb_corr, corr_mat);
% Significant values
plot([0; nb_corr; NaN; 0; nb_corr],...
    [significant_cor(1)*ones(2, 1); NaN; significant_cor(2)*ones(2, 1)], 'k--');
% Limits
xlim([0; nb_corr]);
ylim([min(corr_mat(:)), 1]);
% Labels
stk_xlabel('Lag');
stk_ylabel('Auto-correlations');
stk_title('Auto-correlations of the sampled hyper-parameters');
end

function hand = stk_haario_figure_param(xf, param_name)
% Plot the sampled hyper-parameters

nb_param  = size(xf, 1);
dim_param = size(xf, 2);
[s1, s2] = stk_nb_subplot(dim_param);
%% Figure
hand = stk_figure('Sampled hyper-parameters');
% for each parameters = each sub-plot
for kdp = 1:dim_param
    stk_subplot(s1, s2, kdp);
    % Plot the values
    plot(1:nb_param, xf(:, kdp), 'x');
    % Limits
    xlim([1; nb_param]);
    ylim([min(xf(:, kdp)) - eps, max(xf(:, kdp)) + eps]);
    % Labels
    stk_xlabel('Number');
    stk_ylabel(param_name{kdp});
end
end

function hand = stk_haario_figure_autocorr(corr_mat, param_name, significant_cor)
% Plot correlation for each parameters

nb_corr = size(corr_mat, 1) - 1;
dim_param = size(corr_mat, 2);
[s1, s2] = stk_nb_subplot(dim_param);
%% Figure
hand = stk_figure('Correlations of the chains');
% for each parameters = each sub-plot
for idp = 1:dim_param
    stk_subplot(s1, s2, idp);
    hold on;
    % zeros axis
    plot([0; nb_corr], zeros(2, 1), 'k-');
    % Correlation associated to the parameters
    plot(0:nb_corr, corr_mat(:, idp));
    % Significant values
    plot([0; nb_corr; NaN; 0; nb_corr],...
        [significant_cor(1)*ones(2, 1); NaN; significant_cor(2)*ones(2, 1)], 'k--');
    % Limits
    xlim([0; nb_corr]);
    ylim([min(corr_mat(:, idp)), 1]);
    % Labels
    stk_xlabel('Lag');
    stk_ylabel(param_name{idp});
end
end

%% Find the number of subplot, according to the number of figure
function [s1, s2] = stk_nb_subplot(n)
s2 = ceil(sqrt(n));
s1 = s2 - 1;
if s1*s2 < n
    s1 = s1 + 1;
end
end

%% Find the name of the parameters
function param_name = stk_find_name(dim_param, model)
% This function returns the name of the sampled hyper-parameters
% according to the model

% Default value (if no model)
param_name = arrayfun(@(x)(sprintf('Hyper-parameter[%d]', x)),...
    1:dim_param, 'UniformOutput', false);

if nargin > 1
    % Parameters of the covariance
    if isa(model.param, 'stk_covmodel')
        % If the covariance model is a stk_covmodel,
        % use the parameters and the prior to find the name of the
        % hyper-parameters
        name_param_cov = fieldnames_protomodel(model.param);
        dn = length(name_param_cov);    % Number of covariance parameters
        param_name(1:dn) = name_param_cov;
        
    else % if is not an stk_covmodel, try to find the covariance name
        covariance_name = model.covariance_type;
        if isa(covariance_name, 'function_handle')
            covariance_name = func2str(covariance_name);
        end
        
        if ~ischar(covariance_name)
            % Wrong covariance name
            warning('STK:stk_haario_figures:wrongCovName',...
                'The covariance type is neither a char, nor a function.');
        elseif strcmp(covariance_name, 'stk_materncov_aniso')
            % Case of an internal covariance of STK, where the regularity is
            % sampled
            param_name{1} = 'logsig2';
            param_name{2} = 'logreg';
            param_name(2 + (1:model.dim)) = arrayfun(@(x)(sprintf('logacc[%d]', x)),...
                1:model.dim, 'UniformOutput', false);
            dn = 2 + model.dim;
            
        elseif strcmp(covariance_name, 'stk_materncov_iso')
            param_name{1} = 'logsig2';
            param_name{2} = 'logreg';
            param_name{3} = 'logacc';
            dn = 3;
            
        elseif any(strcmp(covariance_name, {'stk_materncov52_aniso', 'stk_materncov32_aniso',...
                'stk_gausscov_aniso', 'stk_expcov_aniso', 'stk_sphcov_aniso'}))
            % Case of an internal covariance of STK, where the regularity is
            % fixed
            param_name{1} = 'logsig2';
            param_name(1 + (1:model.dim)) = arrayfun(@(x)(sprintf('logacc[%d]', x)),...
                1:model.dim, 'UniformOutput', false);
            dn = 1 + model.dim;
            
        elseif any(strcmp(covariance_name, {'stk_materncov52_iso', 'stk_materncov32_iso',...
                'stk_gausscov_iso', 'stk_expcov_iso', 'stk_sphcov_iso'}))
            param_name{1} = 'logsig2';
            param_name{2} = 'logacc';
            dn = 2;
            
        else
            % Unknown case
            warning('STK:stk_haario_figures:unknownModel', 'Unknown covariance model.');
        end
    end
    
    % Parameters of the noise variance
    if isa(model.lognoisevariance, 'stk_noisemodel')
        % If the covariance model is a stk_noisemodel,
        % use the parameters and the prior to find the name of the
        % hyper-parameters
        name_param_lnv = fieldnames_protomodel(model.lognoisevariance);
        param_name((dn + 1):dim_param) = name_param_lnv;
        
    elseif stk_isnoisy(model)...
            && isnumeric(model.lognoisevariance) && isscalar(model.lognoisevariance)
        % Internal case to STK, where the log-noise variance is
        % homoscedastic
        param_name{dim_param} = 'lognoisevar';
        
    elseif stk_isnoisy(model) && ~isnumeric(model.lognoisevariance)
        % If there is a noise, but not a numeric (standard case of STK),
        % and not a stk_noisemodel ==> unknown case
        warning('STK:stk_haario_figures:unknownModel', 'Unknown log-noise variance model.');
    end
end

end

%% Find the name of the hyper-parameters of a proto-model
function name_param = fieldnames_protomodel(protomodel)

param = protomodel.param;       % parameters
len_par = length(param);        % number of parameters
name_param = cell(len_par, 1);  % Their names

prior = protomodel.prior; % prior
if isa(prior, 'stk_prior_gauss')
    % If it is a Gaussian prior
    variance = diag(prior.covariance);
    is_fixed = (variance == 0);
    % remove the case where the parameters is supposed known
else
    % All are preserved
    is_fixed = false(len_par, 1);
end


na = 0; % Number of filled fields
param_fields = fieldnames(param);   % List of fields
for xpcf = 1:length(param_fields)
    % Find the number of numerical values associated
    value_param = get(param, param_fields{xpcf, 1});
    le_vp = length(value_param);
    
    if le_vp == 1
        % If there is only one
        % Name(numerical parameters) == Name of the fields
        na = na + 1;
        name_param{na, 1} = param_fields{xpcf, 1};
    else
        % Otherwise,
        % Name(numerical parameters) == Name of the fields[number in
        % the field]
        for ke = 1:le_vp
            na = na + 1;
            name_param{na, 1} = sprintf('%s[%d]', param_fields{xpcf, 1}, ke);
        end
        
    end
end

% Remove fixed hyper-parameters ==> not sampled
name_param = name_param(~is_fixed);

end