% PLOT1D_NOISE is a convenient plot function for 1D kriging predictions
% with noise
%
% CALL: H_PLOT = plot1d_noise (H_AXES, XI, ZI, XT, ZT, ZP, ZSIM)
%
% See also: stk_plot1d

function h_plot = plot1d_noise (h_axes, xi, zi, xt, zt, zp, zsim)

%% Check input arguments
if isempty(h_axes) || ~ishandle(h_axes)
    h_axes = gca();
end

has_zt_arg   = (nargin > 4) && (~ isempty (zt));
has_zp_arg   = (nargin > 5) && (~ isempty (zp));
has_zsim_arg = (nargin > 6) && (~ isempty (zsim));

%% Colors and styles
% Interval of uncertainty
proba_unc = 95/100;
try
    kunc = -sqrt(2) * erfcinv(1 + proba_unc);
catch
    % Avoid the use of a package
    kunc = norminv((1 + proba_unc)/2);
end

% Noise
color_noise = [0.25, 0.75, 0.3];	% Green-Cyan
style_noise = {color_noise, 'LineStyle', '-', 'LineWidth', 1,...
    'EdgeColor', color_noise};

% Uncertainty
color_unc = [0.8, 0.8, 0.8];
style_unc = {color_unc, 'LineStyle', '-', 'LineWidth', 1,...
    'EdgeColor', color_unc};

% Prediction
style_pred  = {'LineStyle', '-', 'LineWidth', 3, 'Color', [0.95 0.25 0.3]};

% Sample paths
color_truth = [0.39, 0.47, 0.64];
style_sim = {'LineStyle', '-',  'LineWidth', 1, 'Color', color_truth};

% Real function
style_truth = {'LineStyle', '--', 'LineWidth', 3, 'Color', color_truth};

% Observations
style_obs   = {'LineStyle', 'none', 'Marker', 'o', 'MarkerSize', 6,...
    'Color', 'k', 'MarkerFaceColor', 'k'};

%% Plot the different parts of the graphic
xt = double (xt);

% Shaded area representing pointwise confidence intervals
if has_zp_arg
    xt_fill = [xt; flipud(xt)];
    
    % Plot the noise part
    if any(strcmpi(zp.colnames, 'var_obs'))
        delta_noise = kunc * sqrt (abs (zp.var_obs));
        try
            uncer_noise = [zp.mean - delta_noise; flipud(zp.mean + delta_noise)];
            
            h_noi = fill (h_axes, xt_fill, uncer_noise, style_noise{:});
        catch
            uncer_noise = [zp.mean - delta_noise, 2 * delta_noise];
            
            h_noi = area(h_axes, xt, uncer_noise, 'FaceColor', style_noise{:});
            delete(h_noi(1));   h_noi = h_noi(2);
        end
        % Raise current axis to the top layer, to prevent it
        % from being hidden by the grayed area
        set (gca, 'Layer', 'top');
        
        hold on;
    else
        h_noi = [];
    end
    
    % Plot the uncertainty part
    delta = kunc * sqrt (abs (zp.var));
    try
        uncer = [zp.mean - delta; flipud(zp.mean + delta)];
        
        h_ci = fill (h_axes, xt_fill, uncer, style_unc{:});
    catch
        uncer = [zp.mean - delta, 2 * delta];
        
        h_ci = area (h_axes, xt, uncer, 'FaceColor', style_unc{:});
        delete(h_ci(1));    h_ci = h_ci(2);
    end
    % Raise current axis to the top layer, to prevent it
    % from being hidden by the grayed area
    set (gca, 'Layer', 'top');
    
    hold on;
else
    h_ci = [];
end

% Plot sample paths
if has_zsim_arg
    if isa (zsim, 'stk_dataframe')
        % Prevents automatic creation of a legend by @stk_dataframe/plot
        zsim.colnames = {};
    end
    h_sim = plot (h_axes, xt, double(zsim), style_sim{:});
    hold on;
else
    h_sim = [];
end

% Ground truth
if has_zt_arg
    h_truth = plot (h_axes, xt, double(zt), style_truth{:});
    hold on;
else
    h_truth = [];
end

% Kriging predictor (posterior mean)
if has_zp_arg
    h_pred = plot (h_axes, xt, zp.mean, style_pred{:});
    hold on;
else
    h_pred = [];
end

% Evaluations
if ~ isempty (zi)
    h_obs = plot (h_axes, double(xi), double(zi), style_obs{:});
else
    h_obs = [];
end

hold off;  set (gca, 'box', 'off');

% Return an object only if asked
if nargout > 0
    % Make it possible to recover all the handles easily, if needed
    h_plot.truth = h_truth;
    h_plot.obs   = h_obs;
    h_plot.pred  = h_pred;
    h_plot.ci    = h_ci;
    h_plot.noise = h_noi;
    h_plot.sim   = h_sim;
end

end

