% SET_MODEL_INIT is an extension of stk_model, which initializes a model
% with the priors and the parameters.
%
% CALL: MODEL = set_model_init (COVARIANCE_TYPE, XI, ZI, BND, RANGE_OUTPUT, IS_NOISE)
function model = set_model_init(covariance_type, xi, zi, bnd, range_output, is_noise)

%% Check inputs
dim = size(xi, 2);
if ~isequal( size(zi), [size(xi, 1), 1] )
    stk_error(['zi must be a column matrix with the same number of', ...
        ' lines than xi.'], 'IncorrectSize');
end

% Bounds
if nargin < 4 || isempty(bnd)
    bnd = stk_boundingbox(xi);
else
    if ~isa(bnd, 'stk_hrect')
        % Check that the bounds have 2 lines with the second greater that
        % the first one.
        bnd = stk_hrect(bnd);
    end
    
    if size(bnd, 2) ~= dim
        stk_error('bnd and xi does not have the same dimension.', 'IncorrectSize');
    end
end

% Range of outputs
if nargin < 5 || isempty(range_output)
    range_output = 1.5*range(zi);
end

% is noise ?
if nargin < 6 || isempty(is_noise)
    is_noise = true;
end

%% The model
% Create a model from inputs (check covariance_type)
model = stk_model(covariance_type, dim);
if is_noise
    % Indicate that the log-noise variance must be estimated
    model.lognoisevariance = nan;
end

%% Type of covariance
[have_reg, is_iso] = type_covariance(func2str(model.covariance_type));
% Special case where the covariance is unknown => no prior

add_prior = ~isnan(is_iso) && ~isnan(have_reg);

%% Add prior to the model
if add_prior
    %% Values of prior mean and variance
    % Prior of the model
    
    % log(sigma^2)
    mean_prior_logsig2 = 2*(log(range_output) - log(10));
    var_prior_logsig2 = (log(100)/3)^2;
    
    % log(nu)
    mean_prior_logreg  = log(5/2);
    var_prior_logreg  = (log(5)/3)^2;
    % m[log(nu)] - 3*s[log(nu)] = log(5/2) - log(5) = log(1/2)
    % m[log(nu)] + 3*s[log(nu)] = log(5/2) + log(5) = log(25/2) = log(12.5)
    
    % -log(rho)
    mean_prior_logacc  = log(2./diff(bnd))';
    var_prior_logacc  = (log(10)/3)^2;
    
    % log(lambda)
    mean_prior_lognvar = 2*(log(range_output) - log(10));
    var_prior_lognvar = (log(100)/3)^2;
    
    % Prior
    mean_prior_cov = mean_prior_logsig2;
    var_prior_cov = var_prior_logsig2;
    
    % Add the regularity prior if it is a Matern covariance
    if have_reg
        mean_prior_cov = [mean_prior_cov; mean_prior_logreg];
        var_prior_cov = [var_prior_cov; var_prior_logreg];
    end
    
    % Add the range prior depending if the model is iso- or anisotropic.
    if is_iso
        mean_prior_cov = [mean_prior_cov; mean(mean_prior_logacc)];
        var_prior_cov = [var_prior_cov; var_prior_logacc];
    else % is aniso
        mean_prior_cov = [mean_prior_cov; mean_prior_logacc];
        var_prior_cov = [var_prior_cov; var_prior_logacc*ones(dim, 1)];
    end
    
    
    %% Add priors to model
    % Add priors on covariance parameters
    model.prior.mean = mean_prior_cov;
    model.prior.invcov = diag(1./var_prior_cov);
    
    if is_noise
        % Add noise-prior
        model.noiseprior.mean = mean_prior_lognvar;
        model.noiseprior.var = var_prior_lognvar;
    end
end

%% Add parameters to the model
% Initialize parameters
[param0, lnv0] = stk_param_init(model, xi, zi, bnd, is_noise);
if isinf(lnv0)
    lnv0 = [];
end
% Optimize parameters
[param_opt, lnv_opt] = stk_param_estim(model, xi, zi, param0, lnv0, @stk_param_relik);

% Set optimal parameters
model.param = param_opt;
model.lognoisevariance = lnv_opt;
end

