% STK_HAARIO_PREDICT performs a posterior prediction from data and
% simulated parameters
%
% CALL: ZP = STK_HAARIO_PREDICT(SIM_PARAM, MODEL, XI, ZI, XP)
%
%   returns the posterior mean and variance at XP, of the process described
%   by the model, and with observations (XI, ZI). SIM_PARAM are nb_param
%   samples of the hyper-parameters according to their posterior
%   distribution.
%
% See also: stk_predict, stk_haario_adaptive_metropolis
function zp = stk_haario_predict(sim_param, model, xi, zi, xp)

% Number of prediction points
np = size(xp, 1);
% Number of parameters
nb_param = size(sim_param, 1);

is_noise = stk_isnoisy(model);

% Predictions taking into account on simulated parameters
zp_mean_mean= NaN(np, 1);   % E(E[xi(x)|theta]) = E(E[Z|theta])
zp_mean_var = NaN(np, 1);   % E(Var[xi(x)|theta])
zp_var_mean = NaN(np, 1);   % Var(E[xi(x)|theta]) = Var(E[Z|theta])

if is_noise
    zn_mean_var = NaN(np, 1);   % E(Var[Z|theta])
end

%% Recursive formulation of prediction
for kpar = 1:nb_param
    % For each sampled parameters
    
    % 1: change the parameters on the model
    if is_noise && isscalar(model.lognoisevariance);
        % Assume nb_param_lnv = 1; Do not consider heteroscedastic case
        
        % Change covariance parameter
        model.param(:) = sim_param(kpar, 1:(end - 1));
        % Change noise variance parameter
        model.lognoisevariance(:) = sim_param(kpar, end);
    else
        % nb_param_lnv = 0
        % Change covariance parameter values
        model.param(:) = sim_param(kpar, :);
    end
    
    % 2: Predict knowing the parameters
    zp_k = stk_predict(model, xi, zi, xp);
    if is_noise
        zn_var_k = zp_k.var + exp(model.lognoisevariance);
        
        % 3: Update the first and second moment
        zn_mean_var = stk_mean_recursive(kpar, zn_mean_var, zn_var_k);
    end
    zp_mean_var = stk_mean_recursive(kpar, zp_mean_var, zp_k.var);
    
    [zp_mean_mean, zp_var_mean] = stk_update_recursive(kpar, zp_mean_mean,...
        zp_var_mean, zp_k.mean);
    
end

% Concatenate results on one data-frame
zp = stk_dataframe([zp_mean_mean, zp_var_mean + zp_mean_var], {'mean', 'var'});
% E[xi(x)] = E[Z] = E(E[Z|theta])
% Var[xi(x)] = Var(E[xi(x)|theta]) + E(Var[xi(x)|theta])
if is_noise
    zn = stk_dataframe(zp_var_mean + zn_mean_var, {'var_obs'});
    % Var[Z] = Var(E[Z|theta]) + E(Var[Z|theta])
    zp = [zp, zn];
end

end

%% Recursive formulation of mean and variance
function [m_n, v_n] = stk_update_recursive(n, m_nM1, v_nM1, x_n)


%% Update
v_n = stk_var_recursive (n, v_nM1, m_nM1, x_n);
m_n = stk_mean_recursive(n,        m_nM1, x_n);

end

function m_n = stk_mean_recursive(n, m_nM1, x_n)

% Compute the mean
[n, m_nM1, x_n] = stk_commonsize(n, m_nM1, x_n);

b0 = (n <= 0);
b1 = (n == 1);

% n > 1
m_n = ((n - 1)./n).*m_nM1 + x_n./n;
% n == 1
m_n(b1) = x_n(b1);  % n == 1 ==> mean([x_1, ..., x_n]) = mean([x_1]) = x_1
% n < 1
m_n(b0) = NaN;   % n <= 0 ==> mean([x_1, ... , x_n]) = mean([]) = NaN

end

function v_n = stk_var_recursive(n, v_nM1, m_nM1, x_n)


% Compute the variance

[n, v_nM1, m_nM1, x_n] = stk_commonsize(n, v_nM1, m_nM1, x_n);

b1 = (n <= 1);
b2 = (n == 2);

% n > 2
v_n = ((n - 2)./(n - 1)).*v_nM1 + ((m_nM1 - x_n).^2)./n;
% n == 2
v_n(b2) = ((m_nM1(b2) - x_n(b2)).^2)/2;
% n == 2 => var([x_1, ... , x_n]) = var([x_1, x2]) = (x_1 - x_2)^2/2 =
% (mean([x_1]) - x_2)^2/2
% n < 2
v_n(b1) = NaN;  % n < 2 ==> var([x_1, ... , x_n]) = var([x_1]) = 0/(1 - 1)

end
