% TYPE_COVARIANCE returns properties of a covariance from a covariance name.
%
% CALL: [HAVE_REG, IS_ISO] = type_covariance(COVARIANCE_TYPE)
%
%   analyses the name of the covariance, and returns its properties. The
%   function returns NaN outputs, if it does not recognize the covariance
%   (case of personal covariances).
%   HAVE_REG indicates whether the covariance has a regularity parameter
%   (Matern) or not. IS_ISO indicates whether the covariance is isotropic
%   (IS_ISO = true), anisotropic (IS_ISO = false), or neither (IS_ISO = NaN).
%
%   Remark: if HAVE_REG is nan, then the covariance is not a STK covariance
%           function.
function [have_reg, is_iso] = type_covariance(covariance_type)

if nargin > 1
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

if nargin < 1
    stk_error('Not enough input arguments', 'NotEnoughInputArgs');
end

% From stk_model
if isa (covariance_type, 'function_handle')
    covariance_name = func2str (covariance_type);
elseif ischar (covariance_type)
    covariance_name = covariance_type;
else
    stk_error (['''covariance_type'' should be a function name or a handle to ' ...
        'a function.'], 'TypeMismatch');
end

%% Find properties of the covariance

% Is it a covariance in STK ?
stk_prefix = 'stk_';
if ~strncmp(covariance_name, stk_prefix, length(stk_prefix))
    % If is not a STK covariance, returns NaN (impossible to conclude)
    have_reg = NaN;
    is_iso = NaN;
    return;
end


% Does it have a regularity parameter ?
matern_prefix = 'stk_materncov_';
if strncmp(covariance_name, matern_prefix, length(matern_prefix))
    % If the name begins by 'stk_materncov_', then there is a regularity parameter
    have_reg = true;
else % There is not any
    have_reg = false;
end

% Is an iso- or an anisotropic covariance ?
iso_suffix   = fliplr('_iso');
aniso_suffix = fliplr('_aniso');
if strncmp(fliplr(covariance_name), iso_suffix, length(iso_suffix))
    % If the name finished by '_iso', then it is an isotropic covariance
    is_iso = true;
elseif strncmp(fliplr(covariance_name), aniso_suffix, length(aniso_suffix))
    % If the name finished by '_aniso', then it is an anisotropic covariance
    is_iso = false;
else
    % else, impossible to conclude
    is_iso = NaN;
end

end

% Test
%!shared have_reg, is_iso

% Check input arguments

%!error [have_reg, is_iso] = type_covariance();
%!test  [have_reg, is_iso] = type_covariance('toto');
%!test  [have_reg, is_iso] = type_covariance(@toto);
%!error [have_reg, is_iso] = type_covariance('toto', pi);
%!error [have_reg, is_iso] = type_covariance(pi);

% Check with some covariance

%!test
%! [have_reg, is_iso] = type_covariance('stk_materncov_iso');
%! assert ( have_reg && is_iso );

%!test
%! [have_reg, is_iso] = type_covariance('stk_materncov52_iso');
%! assert ( ~have_reg && is_iso );

%!test
%! [have_reg, is_iso] = type_covariance('stk_materncov_aniso');
%! assert ( have_reg && ~is_iso );

%!test
%! [have_reg, is_iso] = type_covariance('stk_materncov52_aniso');
%! assert ( ~have_reg && ~is_iso );

%!test
%! [have_reg, is_iso] = type_covariance('stk_expcov_aniso');
%! assert ( ~have_reg && ~is_iso );

%!test
%! [have_reg, is_iso] = type_covariance('stk_discretecov');
%! assert ( ~have_reg && isnan(is_iso) );

%!test
%! [have_reg, is_iso] = type_covariance('toto');
%! assert ( isnan(have_reg) && isnan(is_iso) );