% STK_VAR_RECURSIVE computes the unbiased variance with a recursive formula
%
% CALL : V_n = stk_var_recursive(n, V_nM1, M_nM1, X_n)
%
%   computes the variance V_n = var([X_1, ..., X_nM1 , X_n], 0), using the
%   number n, the previous variance V_nM1 = var([X_1, ..., X_nM1], 0),
%   the previous mean M_nM1 = mean([x_1, ..., x_nM1]), and the new value X_n.
%
% See also: stk_update_recursive, stk_var_recursive
function v_n = stk_var_recursive(n, v_nM1, m_nM1, x_n)

if nargin ~= 4
    if nargin < 4
        stk_error('Not enough input arguments', 'NotEnoughInputArgs');
    else
        stk_error('Too many input arguments', 'TooManyInputArgs');
    end
end

%% Compute the variance

[n, v_nM1, m_nM1, x_n] = stk_commonsize(n, v_nM1, m_nM1, x_n);

b1 = (n <= 1);
b2 = (n == 2);

% n > 2
v_n = ((n - 2)./(n - 1)).*v_nM1 + ((m_nM1 - x_n).^2)./n;
% n == 2
v_n(b2) = ((m_nM1(b2) - x_n(b2)).^2)/2;
% n == 2 => var([x_1, ... , x_n]) = var([x_1, x2]) = (x_1 - x_2)^2/2 =
% (mean([x_1]) - x_2)^2/2
% n < 2
v_n(b1) = NaN;  % n < 2 ==> var([x_1, ... , x_n]) = var([x_1]) = 0/(1 - 1)

end

%!shared n, x, m, v
%! x = [0.92, 0.98, 0.36, -2.93, -0.67, -0.81, 0.19, -0.93, 0.13, -2.87];
%! n = length(x);
%! m = 0.75;
%! v = -0.14;

%!error v = stk_var_recursive();
%!error v = stk_var_recursive(n);
%!error v = stk_var_recursive(n, v);
%!error v = stk_var_recursive(n, v, m);
%!test  v = stk_var_recursive(n, v, m, x(n));
%!error v = stk_var_recursive(n, v, m, x(n), pi);

%!test
%! m = nan;
%! v = nan;
%! for k = 1:n
%!  v = stk_var_recursive(k, v, m, x(k));
%!  if k > 1
%!      assert( stk_isequal_tolabs(v, var(x(1:k))) );
%!  end
%!  m = stk_mean_recursive(k, m, x(k));
%! end

%! m = mean(x);
%! v = var(x);
%!assert( isnan(stk_var_recursive(0, v, m, x(n))) );
%!assert( isnan(stk_var_recursive(1, v, m, x(n))) );
%!assert( isequal((x - m).^2/2, stk_var_recursive(2, v, m, x)) );
