% STK_ADD_LEVEL adds levels to a matrix or a data-frame.
%
% CALL: X_LEVEL = stk_add_level(X, LEVEL)
%
%   adds LEVEL to X, a matrix or a data-frame which size is n x dim.
%   LEVEL is considered as a vector of length l. The output X_LEVEL is a
%   (n*l) x (dim + 1) matrix or data-frame, such as the first dim columns
%   are l repetitions of data, and the last column contains the vector
%   LEVEL.
%   
%   For all k in [1, l]:
%   - X_LEVEL[{n*(k - 1) + 1}:{n*k}, 1:dim] = X;
% 	- X_LEVEL[{n*(k - 1) + 1}:{n*k}, (dim + 1)] = LEVEL(k).
%
% EXAMPLE
%   dim = 2; n = 10;
%   box = stk_hrect([5, -13; 9, -10], {'x1', 'x2'});
%   level = stk_dataframe([1.1; 2.02], {'Level'});
%   x = stk_sampling_randunif(n, dim, box);
%   x_level = stk_add_level(x, level);
%
% See also: stk_sampling_nestedlhs, stk_sampling_nesteddesign

function x_level = stk_add_level(x, level)

if nargin > 2
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

if nargin < 2
    stk_error('Not enough input arguments', 'NotEnoughInputArgs');
end

%% Check levels
if size(level, 2) > 1
    level = level(:);   % assert column vector
end
nLevel = size(level, 1);
nx = size(x, 1);

%% Repeat the data
if isa(x, 'stk_dataframe')
    x = stk_dataframe(repmat(x.data, nLevel, 1), x.colnames);
else
    x = repmat(x, nLevel, 1);
end

if isa(level, 'stk_dataframe')
    level = stk_dataframe(kron(level.data, ones(nx, 1)), level.colnames);
else
    level = kron(level, ones(nx, 1));
end

%% Combine data and level
x_level = [x, level];
end

%!shared x, n, dim, box, level, y, colx, colL
%! n = 10; dim = 2; box = [0, 0; 2, 2];
%! x = stk_sampling_randunif (n, dim, box);
%! level = [1, 2; 3, 4];
%! colx = {'Height', 'Weight'};
%! colL = {'Test'};

%!error y = stk_add_level ();
%!error y = stk_add_level (x);
%!test  y = stk_add_level (x, level);
%!error y = stk_add_level (x, level, pi);

%!assert ( isequal(size(y), [size(x, 1)*numel(level), size(x, 2) + 1]) )
%!assert( isa(y, 'stk_dataframe') && isequal(y.colnames, {}))

% Check column names
%! y = stk_add_level (double(x), double(level));
%!assert( isnumeric(y) )
%! y = stk_add_level (stk_dataframe(x, colx), double(level));
%!assert( isa(y, 'stk_dataframe') && isequal(y.colnames, [colx, {''}]))
%! y = stk_add_level (stk_dataframe(x, colx), double(level));
%!assert( isa(y, 'stk_dataframe') && isequal(y.colnames, [colx, {''}]))
%! y = stk_add_level (x, stk_dataframe(level(:), colL));
%!assert( isa(y, 'stk_dataframe') && isequal(y.colnames, [{'', ''}, colL]))
%! y = stk_add_level (stk_dataframe(x, colx), stk_dataframe(level(:), colL));
%!assert( isa(y, 'stk_dataframe') && isequal(y.colnames, [colx, colL]))

% Check output argument

%!test
%! for lev = 1:numel(level),
%!	index = (1:n) + (lev - 1)*n;
%!	assert(isequal(double(y(index, 1:dim)), double(x)));
%!	assert(isequal(double(y(index, dim + 1)), level(lev)*ones(n, 1)));
%! end