% CALL : [lb, ub] = stk_param_getdefaultbounds(promo, xi, zi)
%
% Use for optimization, this function returns the bound of optimizable
% fields. This bounds will define the domain of optimization.
% - promo : the proto_model to optimize;
% - xi : the design of observation points;
% - zi : the observation points.
%
% lb <= promo.param <= ub.
% 
% This is a default function. You can overlaod it, by redefining a
% protomodel.
function [lb, ub] = stk_param_getdefaultbounds (varargin)

switch nargin
    case 3
        % Experimental syntax used this project
        promo = varargin{1};
        xi = varargin{2};
        zi = varargin{3};
    case 4
        % Ordinary STK syntax
        func = varargin{1};
        promo = varargin{2};
        xi = varargin{3};
        zi = varargin{4};
        assert_func_consistency (promo, func);
    otherwise
        error ('WTF ?');
end
    

%% Check size
if size(zi, 2) ~= 1
    stk_error('zi is not a vector', 'IncorrectSize')
end
if size(zi, 1) ~= size(xi, 1)
    stk_error('zi and xi have not the same lenght.', 'IncorrectSize');
end

%% Calculate bounds of optimization
[lb, ub] = stk_param_getdefaultbounds(promo.param, xi, zi);
% Remark : promo.func is not used.

if ~isempty(promo.prior)
    [lb_prior, ub_prior] = stk_param_getdefaultbounds(promo.prior);
    if ~isequal(size(lb_prior), size(lb), size(ub_prior), size(ub))
        stk_error('The parameter and the prior have incompatible dimension.', 'IncorrectDim');
    end
    
    for k = 1:numel(lb)
        if (lb_prior(k) <= ub(k)) && (lb(k) <= ub_prior(k))
            % Compatible bounds:
            % l_p < l   < u   < u_p
            % l_p < l   < u_p < u
            % l   < l_p < u_p < u
            % l   < l_p < u   < u_p
            % Returns middle bounds
            lb(k) = max(lb_prior(k), lb(k));
            ub(k) = min(ub_prior(k), ub(k));
        else
            % Incompatible bounds
            % l   < u   < l_p < u_p
            % l_p < u_p < l   < u
            % Returns large bounds
            lb(k) = min(lb_prior(k), lb(k));
            ub(k) = max(ub_prior(k), ub(k));
        end
    end
    
    
    if any(lb > ub)
        stk_error(['Lower bound higher than upper bound.', ...
            ' Your prior and your parameter return incoherant bounds.'], 'AlgorithmFailure');
    end
end

%% Assert current parameters are in the bounds
vect_param = stk_get_optimizable_parameters(promo);
lb = min(lb, vect_param);
ub = max(ub, vect_param);
end

