% CALL : covmo_multifid = stk_covmodel_multifidelity(func, dim)
%
% CALL : covmo_multifid = stk_covmodel_multifidelity(func, dim, reg0)
%
% CALL : covmo_multifid = stk_covmodel_multifidelity(func, dim, reg0, intercept)
%
% CALL : covmo_multifid = stk_covmodel_multifidelity(func, dim, reg0, intercept, regE)
%
% Create a covariance model with a Matern anisotropic covariance function.
%
% - the covariance function : stk_multifidcov_brown or stk_multifidcov_matern
% - dim (default value : 1) : dimension of the model;
% - reg0 (default value : NaN) : the regularity of the ideal function;
% - intercept (default value : 0) : intercept of the error part;
% - regE (default value : reg0) : the regularity of the error function.
function covmo_multifid = stk_covmodel_multifidelity(func, dim, reg0, intercept, regE)

if nargin > 5
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

%% Default value
if nargin < 1 || isempty(func)
    func = 'stk_multifidcov_brown';
else
    if ~strcmp(func, 'stk_multifidcov_brown') && ~strcmp(func, 'stk_multifidcov_matern')
        warning('STK:stk_covmodel_multifidelity:unknownCovariance',...
            'The covariance function is unknown.');
    end
end

if nargin < 2 || isempty(dim)
    dim = 1;
else
    dim = ceil(dim);
    if dim < 1
        stk_error('Dimension must be a strictly positive parameter.', 'InvalidArgument');
    end
end

if nargin < 3 || isempty(reg0)
    reg0 = NaN;
else
    if reg0 < 0
        stk_error('Regularity must be a positive parameter.', 'InvalidArgument');
    end
    if isinf(reg0)
        stk_error('Infinite value is not allowed for regularity.', 'InvalidArgument');
    end
end

if nargin < 4 || isempty(intercept)
    intercept = 0;
end

if nargin < 5 || isempty(regE)
    regE = reg0;
else
    if regE < 0
        stk_error('Regularity must be a positive parameter.', 'InvalidArgument');
    end
    if isinf(regE)
        stk_error('Infinite value is not allowed for regularity.', 'InvalidArgument');
    end
end

%% parameters
param = stk_param_multifidelity_cov(NaN, NaN, NaN,...
    NaN(dim, 1), NaN(dim, 1),...
    log(reg0), log(regE), intercept);

%% Prior
prior = [];

%% Create the covariance model.
covmo_multifid = stk_covmodel(func, param, prior);
covmo_multifid = class(struct(), 'stk_covmodel_multifidelity', covmo_multifid);
end


