function [covparam, noiseparam] = stk_arcov_matern_param_init ...
    (model, xi, zi, box, do_estim_lnv)

if nargin < 2
    covparam = stk_covmodel_autoregre ();
    noiseparam = -inf;
    return
end

% We expect a particular type of protomodel in model.param
covmo_ar_fid = model.param;
assert (isa (covmo_ar_fid, 'stk_covmodel_autoregre'));

%% default value for arguments 'box'
xi = double(xi);
zi = double(zi);

if nargin < 5
    box = [];
end

if ~ isa (box, 'stk_hrect')
    if isempty (box)
        box = stk_boundingbox (xi);  % Default: bounding box
    else
        box = stk_hrect (box);
    end
end

%% Checking zi
if ~ isequal (size (zi), [size(xi, 1) 1])
    errmsg = 'zi should be a column, with the same number of rows as xi.';
    stk_error (errmsg, 'IncorrectSize');
end
d = size (xi, 2);

%% Ensure model
% Make sure that lognoisevariance is -inf for noiseless models
if ~stk_isnoisy (model)
    if (nargin < 6) || (~do_estim_lnv)
        % Assume a noiseless model
        model.lognoisevariance = - inf;
    else % do_estim_lnv is provided, and is equal to true
        % Assume a noisy model with constant but unknown noise variance
        model.lognoisevariance = nan;
    end
end

% Ensure backward compatiblity with respect to model.order / model.lm
model = stk_model_fixlm (model);

%% Do estim log-variance ?

noiseparam = model.lognoisevariance;
if (isnumeric (noiseparam))  % Backward compatibility
    
    if isscalar(noiseparam)
        if nargin < 6  % do_estim_lnv not provided
            do_estim_lnv = isnan (noiseparam);
        elseif (~ do_estim_lnv) && isnan (noiseparam)
            stk_error (sprintf (['do_estim_lnv is false, but the log-noise ' ...
                'variance is equal to NaN. If you don''t want the log-noise ' ...
                'variance to be estimated, you must provide ' ...
                'values for them!']), 'MissingParameterValue');
        end
        
    else % isvector(lnv)
        % Old-style support for the heteroscedastic case: a numeric vector of
        % log-noise variances was stored in model.lognoisevariance (implicitely
        % associated with the locations xi that would be passed later to
        % stk_noisecov...).
        
        % Noise variance estimation not supported in this case
        if ((nargin >= 6) && do_estim_lnv) || (any (isnan (noiseparam)))
            % FIXME: warn that a more modern way of dealing with the heteroscedastic
            % case is now available in STK
            warning('STK:stk_param_init:newfeatures', ['If you want to ', ...
                'define an heteroscedastic noise with parameters to estimate ',...
                'you should create a stk_noisemodel object.']);
            stk_error (['model.lognoisevariance is non-scalar and contains ' ...
                'nans. Noise variance estimation is not supported in the ' ...
                'heteroscedastic case '], 'InvalidArgument');
        else
            do_estim_lnv = false;
        end
    end
    
else  % Case where model.lognoisevariance is a stk_noisemodel
    
    % Estimation of noise variance ?
    %  * if do_estim_lnv is provided, use it to decide
    %  * if not, estimation occurs when lnv parameters contains NaNs
    lnv_vect = stk_get_optimizable_parameters(noiseparam);
    if nargin < 6  % do_estim_lnv not provided
        do_estim_lnv = any (isnan (lnv_vect));
    elseif (~ do_estim_lnv) && (any (isnan (lnv_vect)))
        stk_error (sprintf (['do_estim_lnv is false, but some parameter ' ...
            'values are equal to NaN. If you don''t want the parameters of ' ...
            'the noise variance model to be estimated, you must provide ' ...
            'values for them!']), 'MissingParameterValue');
    end
end

% Warn when there are useless computation
if (do_estim_lnv) && (nargout < 2)
    warning (['stk_param_init will be computing an estimation of the ' ...
        'variance of the noise, perhaps should you call the function ' ...
        'with two output arguments?']);
end

%% Begin to propose values
covparam = covmo_ar_fid;

param_ar = get(covmo_ar_fid, 'param');
nb_lev = number_levels(param_ar);

% prior = get(covmo_ar_fid0, 'prior');   % the prior is ignored in this function


%% Decompose xi
xi_t = xi(:, d);
xi_x = xi(:, 1:(d - 1));

if any(xi_t < 1) || any(xi_t > nb_lev) || any(xi_t ~= floor(xi_t))
    stk_error('Levels of xi must be integer between 1 and the number of levels', 'InvalidArgument');
end

%% Propose default values for Matern covariance parameters
empty_level = false(nb_lev, 1);
params_0 = cell(nb_lev, 1);
for kc = 1:nb_lev
    idx_kc = (xi_t == kc);
    xi_c_k = xi_x(idx_kc, :);
    if ~isempty(xi_c_k)
        % Get the regularity parameters
        param_k = param_ar.(['param_', num2str(kc)]);
        if isa(param_k, 'stk_param_matern_aniso')
            reg_k = param_k.reg;
        else
            reg_k = NaN;
        end
        % Create a model
        model_k = stk_create_model_matern(false, d - 1, isnan(reg_k), reg_k, do_estim_lnv);
        % Initialize
        [cov_k, ~] = stk_param_init(model_k, xi_c_k, zi(idx_kc, 1),...
            box(:, 1:(d - 1)), do_estim_lnv);
        params_0{kc} = cov_k.param;
    else
        empty_level(kc) = true; % else special feature
    end
end

if any(empty_level)
    for ks = 1:nb_lev
        if empty_level(ks)
            dist = abs((1:nb_lev)' - ks)./(1 - double(empty_level));
            [~, indLev] = min(dist);
            params_0{ks} = params_0{indLev};
            % if there is not any point at level kl, take the parameters
            % of the nearest level
        end
    end
end

%% Propose estimation of the rho_coefficient
rhoCoeff_0 = ones(nb_lev - 1, 1);
for ka = 1:(nb_lev - 1)
    if ~empty_level(ka) && ~empty_level(ka + 1)
        
        idx_ka   = (xi_t == ka);
        idx_kaP1 = (xi_t == (ka + 1));
        zka     = zi(idx_ka, 1);
        zkaP1   = zi(idx_kaP1, 1);
        
        ratio = (1./zka)*zkaP1';
        ratio = ratio(:);
        rhoCoeff_0(ka) = median(ratio);
    end
end

param_ar.params = params_0;
param_ar.rhoCoeff = rhoCoeff_0;

%% Estimate an initial value for noise variance
covparam = set(covparam, 'param', param_ar);
model.param = covparam;

if do_estim_lnv
    % FIXME: We should be calling stk_param_init_lnv here...
    noiseparam = stk_param_init_lnv_ (noiseparam, model, xi, zi);
end

end

