function [covparam, noiseparam] = stk_multifidcov_param_init ...
    (model, xi, zi, box, do_estim_lnv)

if nargin < 2
    covparam = stk_covmodel_multifidelity ('stk_multifidcov_matern', model.dim);
    noiseparam = -inf;
    return
end

% We expect a particular type of protomodel in model.param
covmo_multifid = model.param;
assert (isa (covmo_multifid, 'stk_covmodel_multifidelity'));

if nargin < 2
    error ('Implement me !');
end

%% default value for arguments 'box'
if nargin < 5
    box = [];
end

if ~ isa (box, 'stk_hrect')
    if isempty (box)
        box = stk_boundingbox (xi);  % Default: bounding box
    else
        box = stk_hrect (box);
    end
end

%% Checking zi
if ~ isequal (size (zi), [size(xi, 1) 1])
    errmsg = 'zi should be a column, with the same number of rows as xi.';
    stk_error (errmsg, 'IncorrectSize');
end
dim = size (xi, 2);   % dimension
dimx = dim - 1; % dimension without level

%% Ensure model
% Make sure that lognoisevariance is -inf for noiseless models
if ~stk_isnoisy (model)
    if (nargin < 6) || (~do_estim_lnv)
        % Assume a noiseless model
        model.lognoisevariance = - inf;
    else % do_estim_lnv is provided, and is equal to true
        % Assume a noisy model with constant but unknown noise variance
        model.lognoisevariance = nan;
    end
end

% Ensure backward compatiblity with respect to model.order / model.lm
model = stk_model_fixlm (model);

%% Do estim log-variance ?

noiseparam = model.lognoisevariance;
if (isnumeric (noiseparam))  % Backward compatibility
    
    if isscalar(noiseparam)
        if nargin < 6  % do_estim_lnv not provided
            do_estim_lnv = isnan (noiseparam);
        elseif (~ do_estim_lnv) && isnan (noiseparam)
            stk_error (sprintf (['do_estim_lnv is false, but the log-noise ' ...
                'variance is equal to NaN. If you don''t want the log-noise ' ...
                'variance to be estimated, you must provide ' ...
                'values for them!']), 'MissingParameterValue');
        end
        
    else % isvector(lnv)
        % Old-style support for the heteroscedastic case: a numeric vector of
        % log-noise variances was stored in model.lognoisevariance (implicitely
        % associated with the locations xi that would be passed later to
        % stk_noisecov...).
        
        % Noise variance estimation not supported in this case
        if ((nargin >= 6) && do_estim_lnv) || (any (isnan (noiseparam)))
            % FIXME: warn that a more modern way of dealing with the heteroscedastic
            % case is now available in STK
            warning('STK:stk_param_init:newfeatures', ['If you want to ', ...
                'define an heteroscedastic noise with parameters to estimate ',...
                'you should create a stk_noisemodel object.']);
            stk_error (['model.lognoisevariance is non-scalar and contains ' ...
                'nans. Noise variance estimation is not supported in the ' ...
                'heteroscedastic case '], 'InvalidArgument');
        else
            do_estim_lnv = false;
        end
    end
    
else  % Case where model.lognoisevariance is a stk_noisemodel
    
    % Estimation of noise variance ?
    %  * if do_estim_lnv is provided, use it to decide
    %  * if not, estimation occurs when lnv parameters contains NaNs
    lnv_vect = stk_get_optimizable_parameters(noiseparam);
    if nargin < 6  % do_estim_lnv not provided
        do_estim_lnv = any (isnan (lnv_vect));
    elseif (~ do_estim_lnv) && (any (isnan (lnv_vect)))
        stk_error (sprintf (['do_estim_lnv is false, but some parameter ' ...
            'values are equal to NaN. If you don''t want the parameters of ' ...
            'the noise variance model to be estimated, you must provide ' ...
            'values for them!']), 'MissingParameterValue');
    end
end

% Warn when there are useless computation
if (do_estim_lnv) && (nargout < 2)
    warning (['stk_param_init will be computing an estimation of the ' ...
        'variance of the noise, perhaps should you call the function ' ...
        'with two output arguments?']);
end

%% Begin to propose values
% The covariance model
covparam = covmo_multifid;

% Parameters
param_multifid = get(covmo_multifid, 'param');
if length(param_multifid.logaccI) ~= dimx
    stk_error('The Matern anisotropic parameters do not have the same dimension as xi.',...
        'IncorrectSize');
end
% Prior distributions
prior = get(covparam, 'prior');

%% Default value for intercept
if isnan(param_multifid.intercept)
    % propose a default value for intercept
    if isempty(prior)
        param_multifid.intercept = 0;
    else
        % center of the prior bounds
        idx_intercept = find_numb(param_multifid, 'intercept');
        [lb, ub] = stk_param_getdefaultbounds(prior);
        param_multifid.intercept = (lb(idx_intercept) + ub(idx_intercept))/2;
    end
end

%% Decompose xi
ind_x = 1:dimx;
intercept = param_multifid.intercept;
xi_lev = xi(:, dim);
xi_lev_i = abs(double(xi_lev) - intercept);
levels = unique(xi_lev_i);  % list of levels in increasing order

% List of points without levels
x_ix = NaN(0, dimx);
z_ix = NaN(0, 1);
for lev = 1:length(levels)
    % Extract levels
    idx_lev = (levels(lev) == xi_lev_i);
    
    % Extract inputs
    xi_id = xi(idx_lev, ind_x);
    zi_id = zi(idx_lev, :);
    
    if ~isempty(x_ix)
        
        % Add only points with are not already in the design
        Dlev = stk_dist(xi_id, x_ix);
        mDlev = min(Dlev, [], 2);
        x_ix = cat(1, x_ix, xi_id((mDlev ~= 0), :));
        z_ix = cat(1, z_ix, zi_id((mDlev ~= 0), :));
        
    else % if isempty, add all points
        x_ix = xi_id;
        z_ix = zi_id;
    end
    
end

%% Propose default values for Matern covariance parameters
% Create the covariance model for ideal process
cov_matern0 = stk_covmodel_maternaniso(dimx, param_multifid.regI);
cov_matern0.param.logsig2 = param_multifid.logsig2;
cov_matern0.param.logacc  = param_multifid.logaccI;

% Add potential prior distributions
if ~isempty(prior)
    if isa(prior, 'stk_prior_gauss')
        ind_paramI = 1:(dim+1);
        
        cov_matern0.prior = stk_prior_gauss(prior.mean(ind_paramI, 1),...
            prior.covariance(ind_paramI, ind_paramI));
    else
        warning('Unknown prior distribution');
    end
end

% Initialize using the standard initialization of matern parameters
model_matern0 = stk_model('stk_materncov_aniso', dimx);
model_matern0.param = cov_matern0;
model_matern0.lognoisevariance = nan;  % initialize with noise

model_matern0.lm = stk_lm_constant;% let the mean constant

[covmo_mataniso0, lnv_mataniso0] = stk_param_init ...
    (model_matern0, x_ix, z_ix, box(:, ind_x), true);
model_matern0.param = covmo_mataniso0;
model_matern0.lognoisevariance = lnv_mataniso0;

% Save parameters
param_multifid.logsig2 = covmo_mataniso0.param.logsig2;
param_multifid.logaccI = covmo_mataniso0.param.logacc;
param_multifid.logregI = covmo_mataniso0.param.logreg;

%% Estimate the error part
% Estimating error
zp0 = stk_predict(model_matern0, x_ix, z_ix, xi(:, ind_x));
err = double(zi - zp0.mean);

% Propose initial values for error parameters
cov_maternE = stk_covmodel_maternaniso(dim - 1, param_multifid.regE);
cov_maternE.param.logsig2 = param_multifid.logsig2 + param_multifid.logESR;
cov_maternE.param.logacc  = param_multifid.logaccE;

% Add the potential prior
if ~isempty(prior)
    if isa(prior, 'stk_prior_gauss')
        ind_paramE = (dim + 2):(2*dim + 2);
        
        cov_maternE.prior = stk_prior_gauss(prior.mean(ind_paramE, 1),...
            prior.covariance(ind_paramE, ind_paramE));
        % Index of parameters sigmaE^2
        cov_maternE.prior.mean(1) = prior.mean(ind_paramE(1), 1) +  prior.mean(1, 1);
        
        % Covariance
        prior_covE = cov_maternE.prior.covariance;
        % 1 <=> log(sigma_0^2); ind_paramE(1) = log(G)
        % log(sigma_E^2) = log(sigma_0^2) + log(G)
        prior_covE(1, :) = prior.covariance(1, ind_paramE)...
            + prior.covariance(ind_paramE(1), ind_paramE);
        prior_covE(:, 1) = prior.covariance(ind_paramE, 1)...
            + prior.covariance(ind_paramE, ind_paramE(1));
        prior_covE(1, 1) = prior.covariance(1, 1)...
            + prior.covariance(ind_paramE(1), ind_paramE(1)) + 2*prior.covariance(1, ind_paramE(1));
        cov_maternE.prior.covariance = prior_covE;
    end
end

model_maternE = stk_model('stk_materncov_aniso', dimx);
model_maternE.param = cov_maternE;
if do_estim_lnv
    model_maternE.lognoisevariance = nan;  % initialize with noise
else
    model_maternE.lognoisevariance = log(zp0.var);  % add error uncertainty
end
[covmo_matanisoE, ~] = stk_param_init ...
    (model_maternE, xi(:, ind_x), err, box(:, ind_x), do_estim_lnv);

% Save parameters
% Log(G) = log(sigma_E^2) - log(sigma_0^2)
param_multifid.logESR  = covmo_matanisoE.param.logsig2 - param_multifid.logsig2;
param_multifid.logaccE = covmo_matanisoE.param.logacc;
param_multifid.logregE = covmo_matanisoE.param.logreg;

%% Intialize levels parameters

% List of potential values
logregL_list = 0.5; % fixed at 0.5

len_logaccL_list = 5;   % Number of tested values

% Bound of test values of log(1/rho(L))
box_diameter = diff(box(:, dim));

rhoL_max  = 2 * box_diameter;
rhoL_min  = box_diameter/50;

log_rhoL_max = log(rhoL_max);
log_rhoL_min = log(rhoL_min);
% List of tested values
logrhoL_list = linspace(log_rhoL_min, log_rhoL_max, len_logaccL_list);
logaccL_list = -logrhoL_list;

% List of test values for degree
if isempty(prior)
    logdegr_list = linspace(0.5, 4.5, 11);
    % exp(0.5) =  1.65
    % exp(4.5) = 90.02
else
    % use the prior
    idx_logdegr = find_numb(param_multifid, 'logdegr');
    [lb, ub] = stk_param_getdefaultbounds(prior);
    
    mdegr = (ub(idx_logdegr) + lb(idx_logdegr))/2;
    sdegr = (ub(idx_logdegr) - lb(idx_logdegr))/10;
    % Use 3 sigma intervals
    logdegr_list = linspace(mdegr - 3*sdegr, mdegr + 3*sdegr, 11);
end

%% Put the list as grid
% Repmat parameters of the level evolution part
[logregL_list, logaccL_list, logdegr_list] = ndgrid(logregL_list, logaccL_list, logdegr_list);
logregL_list = logregL_list(:);
logaccL_list = logaccL_list(:);
logdegr_list = logdegr_list(:);

matrix_param = [logaccL_list, logregL_list, logdegr_list];
if ~isempty(prior) && isa(prior, 'stk_prior_gauss')
    idx_level = [find_numb(param_multifid, 'logaccL');
        find_numb(param_multifid, 'logregL');
        find_numb(param_multifid, 'logdegr')];
    
    prior_level = stk_prior_gauss(prior.mean(idx_level), prior.covariance(idx_level, idx_level));
    matrix_param = stk_param_init_adapt_to_prior(prior_level, matrix_param);
    
    
    nb_test = size(matrix_param, 1);
    if nb_test > 50
        % not more than 50 test values
        ind_extract = round(linspace(1, nb_test, 50));
        matrix_param = matrix_param(ind_extract, :);
    end
    
end

%% Initialize parameter search
param_best = NaN;
noimo_best = noiseparam;
aLP_best   = +Inf;

covparam = set(covparam, 'param', param_multifid);

% Try all possible for parameters
for kll = 1:size(matrix_param, 1)
    % Set parameters
    param_k = param_multifid;
    param_k.logaccL = matrix_param(kll, 1);
    param_k.logregL = matrix_param(kll, 2);
    param_k.logdegr = matrix_param(kll, 3);
    covmo_multifid_k = covparam;
    covmo_multifid_k = set(covmo_multifid_k, 'param', param_k);
    
    model.param = covmo_multifid_k;
    
    if do_estim_lnv %% && ~isnumeric(noimo0)
        % FIXME: We should be calling stk_param_init_lnv here...
        model.lognoisevariance = stk_param_init_lnv_ (noiseparam, model, xi, zi);
    else % ~do_estim_lnv ==> nothing to change
        % FIXME: Explain why (~ do_estim_lnv) => noiseless !?!
        model.lognoisevariance = -Inf;
    end
    
    % Now, compute the antilog-psoterior
    aLP = stk_param_mlogpost(model, xi, zi);
    if ~isnan(aLP) && (aLP < aLP_best)
        aLP_best   = aLP;
        param_best = model.param;
        noimo_best = model.lognoisevariance;
    end
end

if isinf (aLP_best)
    errmsg = 'Couldn''t find reasonable parameter values... ?!?';
    stk_error (errmsg, 'AlgorithmFailure');
end

%% Return values
noiseparam = noimo_best;
covparam = param_best;
end
