% STK_GAUSSCOV_ANISO computes the anisotropic Gaussian covariance function
%
% CALL: K = stk_gausscov_aniso (PARAM, X, Y)
%
%   computes  the covariance matrix K  between  the sets of locations  X  and Y,
%   using  the anisotropic Gaussian covariance  function  with parameters PARAM.
%   The output matrix K has size  NX x NY,  where NX is the number of rows  in X
%   and NY  the number of rows in Y.  The vector of parameters must have DIM + 1
%   elements, where DIM is the common number of columns of X and Y:
%
%     * PARAM(1) = log (SIGMA ^ 2), where SIGMA is the standard deviation,
%
%     * PARAM(2:END) = - log (RHO), where RHO is the vector of range parameters.
%
% CALL: dK = stk_gausscov_aniso (PARAM, X, Y, DIFF)
%
%   computes the derivative of the covariance matrix with respect to PARAM(DIFF)
%   if DIFF is between 1 and DIM + 1, or the covariance matrix itself if DIFF is
%   equal to -1  (in which case this is equivalent to stk_gausscov_aniso (PARAM,
%   X, Y)).
%
% CALL: K = stk_gausscov_aniso (PARAM, X, Y, DIFF, PAIRWISE)
%
%   computes the covariance vector  (or a derivative of it if DIFF > 0)  between
%   the sets of locations X and Y.  The output K is a vector of length N,  where
%   N is the common number of rows of X and Y.
%
% See also: stk_sf_gausscorr, stk_gausscov_iso

function kov = stk_gausscov_aniso (par_aniso, x, y, diff, pairwise)

if nargin > 5,
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

persistent x0 y0 xs ys param0 pairwise0 D Kx_cache compute_Kx_cache

% process input arguments
x = double (x);
y = double (y);
if nargin < 4, diff = -1; end
if nargin < 5, pairwise = false; end

% check consistency for the number of factors
dim = size (x, 2);
if size (y, 2) ~= dim,
    stk_error ('xi and yi have incompatible sizes.', 'InvalidArgument');
end

% extract parameters from the "param"
Sigma2 = get(par_aniso, 'sig2');
invRho = get(par_aniso, 'acc');
invRho = invRho(:)';

% check param
if ~isequal(size(invRho), [1, dim])
    stk_error ('xi and the anisotropic parameter have incompatible sizes.', 'InvalidArgument');
end

% check parameter values
if ~ (Sigma2 > 0) || ~ all (invRho >= 0),
    error ('Incorrect parameter value.');
end

% check if all input arguments are the same as before
% (or if this is the first call to the function)
if isempty (x0) || isempty (y0) || isempty (param0) || ...
        ~ isequal ({x, y, [Sigma2, invRho]}, {x0, y0, param0}) || ...
        ~ isequal (pairwise, pairwise0)
    % compute the distance matrix
    xs = bsxfun (@times, x, invRho);
    ys = bsxfun (@times, y, invRho);
    D = stk_dist (xs, ys, pairwise);
    % save arguments for the next call
    x0 = x;  y0 = y;  param0 = [Sigma2, invRho];  pairwise0 = pairwise;
    % recomputation of Kx_cache is required
    compute_Kx_cache = true;
end

if diff == -1,
    % Compute the value (not a derivative)
    kov = Sigma2 * stk_sf_gausscorr (D, -1);
else
    [diff_field, ind] = find_field(par_aniso, diff);
    switch diff_field
        case 'logsig2'
            % Differentiate wrt log (Sigma2)
            kov = Sigma2 * stk_sf_gausscorr (D, -1);
        case 'logacc'
            % Differentiate wrt - log (invRho(ind))
            if compute_Kx_cache || isempty (Kx_cache)
                Kx_cache  = 1 ./ (D + eps) .* (Sigma2 * stk_sf_gausscorr (D, 1));
                compute_Kx_cache = false;
            end
            
            if pairwise,
                kov = ((xs(:, ind) - ys(:, ind)) .^ 2) .* Kx_cache;
            else
                kov = (bsxfun (@minus, xs(:, ind), ys(:, ind)')) .^ 2 .* Kx_cache;
            end
            
        otherwise
            % No derivative ==> return 0
            if pairwise
                kov = zeros(size(x, 1), 1);
            else
                kov = zeros(size(x, 1), size(y, 1));
            end
    end
end
end % function