% CALL : par_aniso = stk_param_aniso(logsig2, logacc)
%
% Build an anisometric parameter.
% This parameter should be used for anisometric covariance :
%   k(x, y) = sig2*c( ||(x - y).*acc|| ).
% - logsig2 : logarithm of the variance at 0 of the covariance. Must be a
% unique numeric;
% - logacc : logarithm of the accuracies of the covariance. Accuracies are
% the inverse of ranges. Must be a vector;
function par_aniso = stk_param_aniso(logsig2, logacc)

%% Accuracy
if nargin < 2 || isempty(logacc)
    logacc = NaN;    % Default value : no value
else
    % In contrast, accuracy must be a unique double.
    if ~isnumeric(logacc)
        try
            logacc = double(logacc);
        catch
            stk_error('''logacc'' must be a double.', 'InvalidType');
        end
    end

    logacc = logacc(:); % convert in vector
end

%% Variance at 0
if nargin < 1 || isempty(logsig2)
    logsig2 = NaN;    % Default value : no value
else
    % Variance at 0 must be a unique double too.
    if ~isnumeric(logsig2)
        try
            logsig2 = double(logsig2);
        catch
            stk_error('''logsig2'' must be a double.', 'InvalidType');
        end
    end

    if numel(logsig2) > 1
        stk_error('There should be only one value for ''logsig2''.', 'IncorrectSize');
    end
end

%% Build the anisometric parameter
par_aniso = struct('logsig2', logsig2, 'logacc', logacc);
par_aniso = class(par_aniso, 'stk_param_aniso', stk_param());

end

