function kov = stk_arcov_matern(ar_cov, x, y, diff, pairwise)
% STK_ARCOV_MATERN computes the auto-regressive covariance for a list of matern
%
% CALL: KOV = stk_arcov_matern(AR_COV, X, Y, DIFF, PAIRWISE)
%
%   Compute the covariance for corrective model:
%       Z_{t+1} = rho_t*Z_t + delta_{t+1}
%   t between 1 and S - 1, delta_t having a Matrn covariance
% All parameters are stocked in AR_COV.
%
% X and Y are two matrix, whose size are : size(X) = [n, d], and
% size(Y) = [m, d]. d - 1 is the spatial dimension of the problem.
% The last column correspond to the level of the point :
% X(i, :) is at the level t, if and only if, X(i, d) = t
%
% DIFF allows to change the kind of computations. If DIFF = -1, the
% AR_COV will be compute. But, if 1 <= DIFF <= nbParam, (with nbParam =
% length(ko_par(:)) ), the function returns the derivative of the
% covariance according to the diff-th parameter (used for parameter
% estimation).
% AR_COV(:) = [AR_COV.scales(:);
%                AR_COV.cor_param(:);];
% DIFF is an optionnal argument, whose default value is -1.
%
% PAIRWISE is an optionnal boolean, which allows to chose the kind of
% computation.
% PAIRWISE == false (default value) => the output KOV will be a n*m matrix:
%       KOV = (kov(x_i, y_j));
% PAIRWISE == true => the output KOV will be a n*1 vector:
%       Kov = (kov(x_i, y_i));
% in this case, X and Y must have the same size.


if nargin > 5,
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

persistent x_X0 y_X0 x_t0 y_t0
persistent param_list0 KD0 dKD0
persistent rhoCoeff0 P0 dP0
persistent pairwise0
% KD0 is a cubix (KD0 : Kovariance delta)
% d for derivative (along each parameter)

% process input arguments
x = double (x);
y = double (y);
if nargin < 4, diff = -1; end
if nargin < 5, pairwise = false; end

dim = size(x, 2);
if size(y, 2) ~= dim
    stk_error('xi and yi have incompatible sizes.', 'InvalidArgument')
end


x_t = x(:, dim);
y_t = y(:, dim);

x_X = x(:, 1:(dim - 1));
y_X = y(:, 1:(dim - 1));
nb_lev = number_levels(ar_cov);

if any(x_t < 1) || any(x_t > nb_lev) || any(x_t ~= floor(x_t))
    stk_error('Levels of x must be integer between 1 and the number of levels', 'InvalidArgument');
end

if any(y_t < 1) || any(y_t > nb_lev) || any(y_t ~= floor(y_t))
    stk_error('Levels of y must be integer between 1 and the number of levels', 'InvalidArgument');
end

if diff ~= floor(diff)
    stk_error ('''diff'' parameter must be an interger.', 'InvalidArgument');
end

max_lev = min(max(x_t), max(y_t));  % Maximal level between x and y

mat_params = ar_cov.params;

if any(cellfun(@(x)(~isa(x, 'stk_param_matern_aniso')), mat_params))
    stk_error(['To use this covariance, ',...
        'all parameters must be Matern anistropic covariance parameters'],...
        'InvalidArgument');
end

param_list = stk_get_optimizable_parameters(ar_cov);
% The parameters from 1 to nb_lev - 1 corresponds to rhoCoeff
if length(param_list) ~= (nb_lev*(dim + 2) - 1)
    % length(param_list) ~= (nb_lev - 1 + nb_lev*(dim - 1 + 2))
    stk_error('The list of parameters cannot fit the dimension', 'IncorrectSize');
end
param_list = param_list(nb_lev:((nb_lev*(dim + 2) - 1)));

%% Size of the covariance matrix
siz1 = size(x, 1);
if ~pairwise
    siz2 = size(y, 1);
else
    if size(y, 1) ~= siz1
        stk_error('Pairwise is true, but xi and yi do not have the same size.',...
            'InvalidArgument');
    end
    siz2 = 1;
end

%% Compute Matern covariance
if isempty(x_X0) || isempty(x_t0) || isempty(y_X0) || isempty(y_t0)...
        || isempty(param_list0) || isempty(pairwise0)...
        || ~isequal(x_X, x_X0) || ~isequal(x_t, x_t0)...
        || ~isequal(y_X, y_X0) || ~isequal(y_t, y_t0)...
        || ~isequal(param_list, param_list0)...
        || ~isequal(pairwise, pairwise0)...
        || ( (diff > 0) && isempty(dKD0) )
    
    % Initialize matrices
    KD0 = zeros(siz1, siz2, max_lev);
    if diff > 0 %if derivative is asked
        dKD0 = zeros(siz1, siz2, max_lev, 1 + dim);
        % 2 + (dim - 1)
    else
        dKD0 = [];
    end
    
    for kl = 1:max_lev
        if ~pairwise
            idx_kl = find(x_t >= kl);
            idy_kl = find(y_t >= kl);
            IDI_kl = idx_kl;
            IDJ_kl = idy_kl;
        else    %if pairwise
            IDI_kl = find( (x_t >= kl).*(y_t >= kl) );
            IDJ_kl = 1;
            idx_kl = IDI_kl;
            idy_kl = IDI_kl;
        end     %end if pairwise
        
        KD0(IDI_kl, IDJ_kl, kl) = stk_materncov_aniso(mat_params{kl},...
            x_X(idx_kl, :), y_X(idy_kl, :), -1, pairwise);
        if diff > 0
            for lp = 1:(dim + 1)
                dKD0(IDI_kl, IDJ_kl, kl, lp) = stk_materncov_aniso(mat_params{kl},...
                    x_X(idx_kl, :), y_X(idy_kl, :), lp, pairwise);
            end %end for each parameter
        end %end diff is asked
    end %end for each level
    
    param_list0 = param_list;
    
end

%% Computation of scaling parameters
rhoCoeff = ar_cov.rhoCoeff;

if isempty(x_t0) || isempty(y_t0)...
        || isempty(rhoCoeff0) || isempty(pairwise0)...
        || ~isequal(x_t, x_t0) || ~isequal(y_t, y_t0)...
        || ~isequal(rhoCoeff, rhoCoeff0)...
        || ~isequal(pairwise, pairwise0)...
        || ( (diff > 0) && isempty(dP0) )
    
    
    % Initialize matrices
    P0 = ones(siz1, siz2, max_lev);
    if diff > 0 %if derivative is asked
        dP0 = zeros(siz1, siz2, max_lev, nb_lev - 1);
        % 2 + (dim - 1)
    else
        dP0 = [];
    end
    
    if all(rhoCoeff ~= 0)
        scales = [1; cumprod(rhoCoeff)];
        if ~pairwise
            prod_scales = scales(x_t)*scales(y_t)';
        else
            prod_scales = scales(x_t).*scales(y_t);
        end
        
        for kle = 1:max_lev
            P0(:, :, kle) = prod_scales/(scales(kle)^2);
            % If derivative is asked
            if diff > 0
                for lle = 1:(max_lev - 1)
                    % Power of derivation (0, 1 or 2)
                    if ~pairwise
                        select = double( repmat((lle < x_t), 1, length(y_t)) ...
                            + repmat((lle < y_t)', length(x_t), 1) );
                    else
                        select = double( (lle < x_t) + (lle < y_t) );
                    end % select is 0, 1 or 2
                    
                    dP0(:, :, kle, lle) = (select.*P0(:, :, kle))/rhoCoeff(lle);
                end
            end
            
        end
        
    else % there is at least one zero rhoCoeff, division is impossible
        % stk_error('The programmer did not program the case where rho equals to 0', 'LazyProgrammer');
        for i = 1:siz1
            for j = 1:siz2
                % For each element of the matrix (pointwise fill)
                
                % Select the level of x and y
                t1 = x_t(i);
                if ~pairwise
                    t2 = y_t(j);
                else
                    t2 = y_t(i);
                end
                
                for k = 1:max_lev
                    list_rho1 = rhoCoeff(k:(t1 - 1));
                    list_rho2 = rhoCoeff(k:(t2 - 1));
                    
                    % with prod([]) = 1
                    part1 = prod(list_rho1);
                    part2 = prod(list_rho2);
                    P0(i, j, k) = part1*part2;
                    % If derivative is asked
                    if diff > 0
                        for l = k:(max_lev - 1)
                            % derivative of part1
                            if l < t1
                                list_diff = list_rho1;
                                list_diff(l - k + 1) = 1;
                                c1 = prod(list_diff);
                            else
                                c1 = 0;
                            end
                            % Derivative of part2
                            if l < t2
                                list_diff = list_rho2;
                                list_diff(l - k + 1) = 1;
                                c2 = prod(list_diff);
                            else
                                c2 = 0;
                            end
                            
                            dP0(i, j, k, l) = c1*part2 + part1*c2;
                        end
                    end
                end
            end
        end
    end
    rhoCoeff0 = rhoCoeff;
end

x_X0 = x_X;
y_X0 = y_X;
x_t0 = x_t;
y_t0 = y_t;
pairwise0 = pairwise;

%% Compute the covariance
kov = zeros(siz1, siz2);
if diff == -1 %standard covariance
    
    for kle = 1:size(KD0, 3)
        kov = kov + P0(:, :, kle).*KD0(:, :, kle);
    end
    
elseif (0 < diff) && (diff <= (nb_lev - 1))
    % derivative along scale parameter
    for kdl = 1:min(diff, size(KD0, 3))
        kov = kov + dP0(:, :, kdl, diff).*KD0(:, :, kdl);
    end
    
elseif ((nb_lev - 1) < diff) && (diff <= (nb_lev*(dim + 2) - 1))
    % derivative along correction parameter
    diff = diff - nb_lev + 1;
    [idx_param, lev] = ind2sub([dim + 1, nb_lev], diff);
    
    if lev <= size(dKD0, 3)
        kov = P0(:, :, lev).*dKD0(:,:, lev, idx_param);
    end % else, return 0
    
else
    stk_error ('Incorrect value for the ''diff'' parameter.', 'InvalidArgument');
end
end

