% CALL : ar_cov = stk_param_ar_cov(rho_vect, list_param)
%
%   builds a parameter for auto-regressive covariance.
%   - rho_vect is a vector of S - 1 coefficients between each level
%   - list_param is a list of S matern anisotropic covariance parameters
%   S > 2
function ar_cov = stk_param_ar_cov(rho_vect, list_param)

if nargin > 3
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

% Default value for rho vector
if (nargin < 1) || isempty(rho_vect)
    if (nargin < 2) || isempty(list_param)
        rho_vect = NaN;
    else
        rho_vect = NaN(1, numel(list_param) - 1);
    end
end

% Defalt value for list of parameters
if (nargin < 2) || isempty(list_param)
    list_param = repmat({stk_param_matern_aniso()}, 1, numel(rho_vect) + 1);
end

%% Check inputs

%  correlation factor must be a unique double.
if ~isnumeric(rho_vect)
    try
        rho_vect = double(rho_vect);
    catch
        stk_error('''rho_vect'' must be a double.', 'InvalidType');
    end
end
rho_vect = rho_vect(:);	% Assert column vector
nb_lev = length(rho_vect) + 1;

% List of parameters
if ~iscell(list_param)
    stk_error('list_param must be a cell array of parameters',...
        'InvalidArgument');
end

list_param = list_param(:);	% Assert column cell arrays.
if length(list_param) ~= nb_lev
    stk_error('Cell of parameters must have S parameters, so must have a length equal to S',...
        'IncorrectSize');
end

if any(cellfun(@(x)(~isa(x, 'stk_param')), list_param))
    stk_error('Parameters must be covariance parameters', 'InvalidArgument');
end

%% Build the auto-regressive covariance parameters

ar_cov = struct('rhoCoeff', rho_vect,...
    'params', {list_param});
ar_cov = class(ar_cov, 'stk_param_ar_cov', stk_param());

end

