% CALL : par_bro = stk_param_brownian(logsig2, logdegr, intercept)
%
% Build a Brownian covariance parameter, use for covariance function of the
% form:
%   k(t, t') = sig2*min{|t - intercept|, |t' - intercept|}^degr
%
% - logsig2 : logarithm of the variance at 0;
% - logdegr : logarithm of degree of the Brownian.
% - intercept : t-intercept of the axis. Default value: 0. 
%
% Note : the true Brownian motion have a degree equal to 1, and an
% intercept equal to 0.
%
% Note : the Brownian covariance is defined only for 1D data
function par_bro = stk_param_brownian(logsig2, logdegr, intercept)

if nargin > 3
    stk_error('Too many input argument.', 'TooManyInputArgs');
end

%% Intercept
if nargin < 2 || isempty(intercept)
    intercept = 0;    % Default value : 0
else
    if ~isnumeric(intercept)
        try
            intercept = double(intercept);
        catch
            stk_error('''intercept'' must be a double.', 'InvalidType');
        end
    end

    if numel(intercept) > 1
        stk_error('There should be only one value for ''intercept''.', 'IncorrectSize');
    end
end
% NaN and Inf value are tolerated, but they should be fixed before usin
% gthe default covariance.
    
%% Degree
if nargin < 2 || isempty(logdegr)
    logdegr = NaN;    % Default value : no value
else
    if ~isnumeric(logdegr)
        try
            logdegr = double(logdegr);
        catch
            stk_error('''logdegr'' must be a double.', 'InvalidType');
        end
    end

    if numel(logdegr) > 1
        stk_error('There should be only one value for ''logdegr''.', 'IncorrectSize');
    end
end

%% Variance at 0
if nargin < 1 || isempty(logsig2)
    logsig2 = NaN;    % Default value : no value
else
    % Variance at 0 must be a unique double too.
    if ~isnumeric(logsig2)
        try
            logsig2 = double(logsig2);
        catch
            stk_error('''logsig2'' must be a double.', 'InvalidType');
        end
    end

    if numel(logsig2) > 1
        stk_error('There should be only one value for ''logsig2''.', 'IncorrectSize');
    end
end

%% Build the parameter
par_bro = struct('logsig2', logsig2, 'logdegr', logdegr, 'intercept', intercept);
par_bro = class(par_bro, 'stk_param_brownian', stk_param());

end

