%STK_MULTIFIDCOV_MATERN
%
% CALL : kov = stk_multifidcov_matern(mf_cov, x, y, diff, pairwise)
%
% Compute the multi-fidelity covariance with a level degression described
% by a non-stationary Matern covariance.
%   k( (x;t) , (y;s) ) = k0(x, y) + (|t - O|*|s - O|)^L * kT(t, s) *kE(x, y)
%
% - mf_cov : a multi-fidelity covariance;
% - x, y : 2 matrices with same number of columns, itself greater than 2.
%   Their last column represents the level of fidelity;
% - diff (default value : -1) : if diff == -1, return the standard
% covariance. Else, return the derivative of the covariance with respect to
% the diff-th parameter of mf_cov.
% - pairwise (default value : false) : if true, return a vector of length
% the number of lines of x and y :
%       kov = (k(xi, yi)), 1 <= i <= length(x) == length(y).
% Else, return a matrix of size length(x) x length(y) :
%       kov = (k(xi, yj)), 1 <= i <= length(x); 1 = j <= length(y).
function kov = stk_multifidcov_matern(mf_cov, x, y, diff, pairwise)

if nargin > 5,
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

persistent x0 y0 t0 s0 pairwise0 vect_paramI_0 vect_paramE_0 vect_paramL_0
persistent kov_I dkov_I_dpar kov_E dkov_E_dpar kov_L dkov_L_dpar

%% process input arguments
x = double (x);
y = double (y);
if nargin < 4, diff = -1; end
if nargin < 5, pairwise = false; end

%% Check consistency for inputs
dim = size (x, 2);
if dim < 2
    stk_error('The covariance accepts only multi-columns matrices.', 'IncorrectSize');
end

if (size (y, 2) ~= dim),
    stk_error ('x and y have incompatible sizes.', 'IncorrectSize');
end

x_x = x(:, 1:(dim - 1));
y_x = y(:, 1:(dim - 1));

x_l = x(:, dim);
y_l = y(:, dim);

%% Get parameters
Sig2 = get(mf_cov, 'sig2');
ESR  = get(mf_cov, 'ESR');

paramI = mf_cov.param_Ideal;
paramI.sig2 = 1.0;
paramE = mf_cov.param_Error;
paramE.sig2 = 1.0;
paramL = mf_cov.param_Level;
if ~isa(paramL.param, 'stk_param_aniso')...
        && ~isa(paramL.param, 'stk_param_iso')
    stk_error(['Too use ''stk_multifidcov_matern'', ',...
        'the non-stationary part must be a Matern covariance.'], 'InvalidArgument');
end

if length(paramL.logacc) > 1
    stk_error(['There is only one level paramater.',...
        'Non-stationary part is bad defined.'], 'IncorrectSize');
end

paramL.logsig2 = 0.0;

vect_paramI = [paramI.logacc; paramI.logreg];
vect_paramE = [paramE.logacc; paramE.logreg];

if isa(paramL.param, 'stk_param_matern_iso') ||...
        isa(paramL.param, 'stk_param_matern_aniso')
    vect_paramL = [paramL.logacc; paramL.logreg; paramL.logdegr; paramL.intercept];
    diff_logreg = true;
else
    vect_paramL = [paramL.logacc; log(0.5); paramL.logdegr; paramL.intercept];
    diff_logreg = false;
end

%% Compute matrices

% Ideal part
if isempty(x0) || isempty(y0) || isempty(pairwise0)... First call to the function
        || ~isequal([x_x; y_x], [x0; y0]) || ~isequal(pairwise, pairwise0)... Inputs are not equal
        || ~isequal(vect_paramI, vect_paramI_0)... Parameters differ
        || ( (diff > 0) && isempty(dkov_I_dpar) )   % Derivative is aked, but is not available
    
    kov_I = stk_materncov_aniso(paramI, x_x, y_x, -1, pairwise);
    dkov_I_dpar = [];
    
    if diff > 0 %if derivative is asked
        dkov_I_dpar = NaN(size(kov_I, 1), size(kov_I, 2), dim);
        listnumb = (1:(dim + 1));
        listnumb(find_numb(paramI, 'logsig2')) = [];
        %No derivativation along sigma2
        
        for kdk = 1:dim
            dkov_I_dpar(:, :, kdk) = stk_materncov_aniso(paramI, x_x, y_x,...
                listnumb(kdk), pairwise);
        end
    end
    
    vect_paramI_0 = vect_paramI;
end


% Error part
if isempty(x0) || isempty(y0) || isempty(pairwise0)...
        || ~isequal([x_x; y_x], [x0; y0]) || ~isequal(pairwise, pairwise0)...
        || ~isequal(vect_paramE, vect_paramE_0)...
        || ( (diff > 0) && isempty(dkov_E_dpar) )
    
    if isequal(vect_paramE, vect_paramI)
        % Avoid useless computation
        kov_E = kov_I;
        dkov_E_dpar = dkov_I_dpar;
    else
        
        kov_E = stk_materncov_aniso(paramE, x_x, y_x, -1, pairwise);
        dkov_E_dpar = [];
        
        if diff > 0 %if derivative is asked
            dkov_E_dpar = NaN(size(kov_E, 1), size(kov_E, 2), dim);
            listnumb = (1:(dim + 1));
            listnumb(find_numb(paramE, 'logsig2')) = [];
            %No derivativation along sigma2
            
            for kdk = 1:dim
                dkov_E_dpar(:, :, kdk) = stk_materncov_aniso(paramE, x_x, y_x,...
                    listnumb(kdk), pairwise);
            end
        end
    end
    
    vect_paramE_0 = vect_paramE;
end

% Level part
if isempty(t0) || isempty(s0) || isempty(pairwise0)...
        || ~isequal([x_l; y_l], [t0; s0]) || ~isequal(pairwise, pairwise0)...
        || ~isequal(vect_paramL, vect_paramL_0)...
        || ( (diff > 0) && isempty(dkov_L_dpar) )
    
    kov_L = stk_materncov_nonstat(paramL, x_l, y_l, -1, pairwise);
    dkov_L_dpar = [];
    if diff > 0 %if derivative is asked
        if diff_logreg
            dkov_L_dpar = NaN(size(kov_E, 1), size(kov_E, 2), 3);
            dkov_L_dpar(:, :, 3) = stk_materncov_nonstat(paramL, x_l, y_l,...
                find_numb(paramL, 'logreg'), pairwise);
        else
            dkov_L_dpar = NaN(size(kov_E, 1), size(kov_E, 2), 2);
        end
        
        dkov_L_dpar(:, :, 1) = stk_materncov_nonstat(paramL, x_l, y_l,...
            find_numb(paramL, 'logacc'), pairwise);
        
        dkov_L_dpar(:, :, 2) = stk_materncov_nonstat(paramL, x_l, y_l,...
            find_numb(paramL, 'logdegr'), pairwise);
    end
    
    
    vect_paramL_0 = vect_paramL;
end

% Save data
x0 = x_x;
t0 = x_l;
y0 = y_x;
s0 = y_l;
pairwise0 = pairwise;

%% Compute Covariance
if diff == -1
    kov = Sig2*(kov_I + ESR*kov_L.*kov_E);
else
    [diff_field, index_diff] = find_field(mf_cov, diff);
    switch diff_field
        case 'logsig2'
            kov = Sig2*(kov_I + ESR*kov_L.*kov_E);
        case 'logacc0'
            kov = Sig2*dkov_I_dpar(:, :,...
                find_numb(paramI, 'logacc', index_diff) - 1);
        case 'logreg0'
            kov = Sig2*dkov_I_dpar(:, :,...
                find_numb(paramI, 'logreg') - 1);
        case 'logESR'
            kov = (Sig2*ESR)*kov_L.*kov_E;
        case 'logaccE'
            kov = (Sig2*ESR)*kov_L.*dkov_E_dpar(:, :,...
                find_numb(paramE, 'logacc', index_diff) - 1);
        case 'logregE'
            kov = (Sig2*ESR)*kov_L.*dkov_E_dpar(:, :,...
                find_numb(paramE, 'logreg') - 1);
        case 'logaccL'
            kov = (Sig2*ESR)*dkov_L_dpar(:, :, 1).*kov_E;
        case 'logregL'
            if diff_logreg
                kov = (Sig2*ESR)*dkov_L_dpar(:, :, 3).*kov_E;
            else
                kov = zeros(size(kov_E));
            end
        case 'logdegr'
            kov = (Sig2*ESR)*dkov_L_dpar(:, :, 2).*kov_E;
        case 'intercept'
            % No derivative ==> return 0
            if pairwise
                kov = zeros(size(x, 1), 1);
            else
                kov = zeros(size(x, 1), size(y, 1));
            end
            
        otherwise
            % No derivative ==> return 0
            if pairwise
                kov = zeros(size(x, 1), 1);
            else
                kov = zeros(size(x, 1), size(y, 1));
            end
    end
    
end

end