%stk_materncov_nonstat is the non-stationnary Matern covariance
% 
% CALL : kov = stk_materncov_nonstat(nonstat_cov, x, y)
%
% CALL : kov = stk_materncov_nonstat(nonstat_cov, x, y, diff)
%
% CALL : kov = stk_materncov_nonstat(nonstat_cov, x, y, diff, pairwise)
%
% Compute the non-stationnary Matern covariance with x and y, 1D vector,
% and nonstat_cov a stk_param_nonstatvar_cov.
%
% - nonstat_cov : a stk_param_nonstatvar_cov object. Its parameter must be
% at least a stk_param_iso or stk_param_aniso. If it is a
% stk_param_matern_iso or stk_param_matern_aniso, the regularity is defined
% by the parameter, else, its 0.5;
% - x, y : vectors where covariance must be estimated.
% - diff (default value : -1) : if diff == -1, return the standard
% covariance. Else, return the derivative of the covariance with respect to
% the diff-th parameter of par_bro.
% - pairwise (default value : false) : if true, return a vector of length
% the length of x and y : kov = (k(xi, yi)), 1 <= i <= length(x) == length(y).
% Else, return a matrix of size length(x) x length(y) :
%   kov = (k(xi, yj)), 1 <= i <= length(x); 1 = j <= length(y).
%
% The non-stationnary Matern covariance is define for 1D data :
%   k(x, y) = sig2*[(|x - intercept|*|y - intercept|)^degree]*M_{reg}(|x -
%   y)*a)
function kov = stk_materncov_nonstat(nonstat_cov, x, y, diff, pairwise)

if nargin > 5,
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

persistent x0 y0 t0 s0 degree0 pairwise0 logreg0 logacc0
persistent Kov0 dKov0_dacc dKov0_dreg TS0 TS0_L lnTS0

%% process input arguments
x = double (x);
y = double (y);
if nargin < 4, diff = -1; end
if nargin < 5, pairwise = false; end

%% check consistency for inputs
dim = size (y, 2);
if dim ~= 1
    stk_error('The covariance accepts only 1D vectors.', 'IncorrectSize');
end

if (size (x, 2) ~= dim),
    stk_error ('x and y have incompatible sizes.', 'IncorrectSize');
end

%% Parameter
if isa(nonstat_cov.param, 'stk_param_aniso')
    if length(nonstat_cov.param) > 3
        stk_error(['The covariance is defined for 1D vectors,'...
            ' but your parameter is defined for several dimensions.'], 'IncorrectDim');
    end
elseif ~isa(nonstat_cov.param, 'stk_param_iso')
    stk_error(['The parameter of nonstat_cov must be a stk_param_iso or'...
            ' stk_param_aniso object.'], 'InvalidArgument');
end

Sigma2 = get(nonstat_cov, 'sig2');
logacc = get(nonstat_cov, 'logacc');
if isa(nonstat_cov.param, 'stk_param_matern_aniso')...
    || isa(nonstat_cov.param, 'stk_param_matern_iso')
    logreg = get(nonstat_cov, 'logreg');
    compute_diff_reg = true;
else
    logreg = log(0.5);
    compute_diff_reg = false;
end

degree = get(nonstat_cov, 'degr');
interc = get(nonstat_cov, 'intercept');

t = abs(x - interc);
s = abs(y - interc);    % transform the values

%% Compute useful data
% check if all input arguments are the same as before
% (or if this is the first call to the function)
if isempty (x0) || isempty (y0) || isempty (logacc0) || isempty(logreg0) ...
        || ~ isequal([x; y], [x0; y0]) ...
        || ~ isequal([logacc; logreg], [logacc0; logreg0])...
        || ~ isequal(pairwise, pairwise0)...
        || (diff > 1 && isempty(dKov0_dacc))
    
    % Build a parameter
    mat_iso = stk_param_matern_iso(log(1.0), logacc, logreg);
    
    % compute the distance matrix
    Kov0 = stk_materncov_iso(mat_iso, x, y, -1, pairwise);
    if diff > 1
        diff_acc   = find_numb(mat_iso, 'logacc');
        dKov0_dacc = stk_materncov_iso(mat_iso, x, y, diff_acc, pairwise);
        if compute_diff_reg
            diff_reg   = find_numb(mat_iso, 'logreg');
            dKov0_dreg = stk_materncov_iso(mat_iso, x, y, diff_reg, pairwise);
        end
    else
        dKov0_dacc = [];
        dKov0_dreg = [];
    end
    
    % save arguments for the next call
    logacc0 = logacc;
    logreg0 = logreg;
    
    x0 = x;
    y0 = y;
end

compute_product = (isempty (t0) || isempty (s0)...
    || ~ isequal([t; s], [t0; s0]) || ~ isequal (pairwise, pairwise0));

if compute_product || isempty (degree0) || ~ isequal(degree, degree0)
    
    if compute_product
        if ~pairwise
            TS0 = (t*s');
        else%pairwise == true
            TS0 = (t.*s);
        end
        lnTS0 = log(TS0);
        lnTS0(isinf(lnTS0)) = 0;
        
        t0 = t;
        s0 = s;
    end
    TS0_L = TS0.^degree;
    
    % save arguments for the next call
    degree0 = degree;
end

pairwise0 = pairwise;

%% Compute covariance
if diff == -1
    kov = Sigma2*TS0_L.*Kov0;
else
    diff_field = find_field(nonstat_cov, diff);
    switch diff_field
        case 'logsig2'
            % Diff wrt log(sig^2)
            kov = Sigma2*TS0_L.*Kov0;
            
        case 'logacc'
            % Diff wrt log(1/rho)
            kov = Sigma2*TS0_L.*dKov0_dacc;
            
        case 'logreg'
            if compute_diff_reg
                kov = Sigma2*TS0_L.*dKov0_dreg;
            else
                % No derivative ==> return 0
                if pairwise
                    kov = zeros(size(x, 1), 1);
                else
                    kov = zeros(size(x, 1), size(y, 1));
                end
            end
            
        case 'logdegr'
            % diff wrt to log(degree)
            kov = (Sigma2*degree)*lnTS0.*TS0_L.*Kov0;
            
        case 'intercept'
            % No derivative wrt to intercept ==> return 0
            if pairwise
                kov = zeros(size(x, 1), 1);
            else
                kov = zeros(size(x, 1), size(y, 1));
            end
            
        otherwise
            % No derivative ==> return 0
            if pairwise
                kov = zeros(size(x, 1), 1);
            else
                kov = zeros(size(x, 1), size(y, 1));
            end
    end
end
end

