%STK_PARAM_INIT_ADAPT_TO_PRIOR is used by stk_param_init to transform a
%list of tested parameters to a list which fit better the prior
%distribution.
%
% CALL : list_param_adapted = stk_param_init_adapt_to_prior(prior, list_param)
%
% - prior : a log-prior object;
% - list_param : a matrix of size (number of parameters to test) x (length
% of parameters) containing the default list of parameters to test in
% stk_param_init
%
% The size(list_param, 2) must fit the dimension of the prior.
%
% * list_param_adapted : a matrix of size (number of parameters to test) x
% (length of parameters), whose parameters fit "better" the prior
% distribution.
%
% This function returns a default behavior, you can program a best one for
% your personal prior.
function list_param_adapted = stk_param_init_adapt_to_prior(prior, list_param)

if nargin > 2
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

if nargin < 2
    stk_error('Too few input arguments', 'TooFewInputArgs');
end

%% Get default bounds of the prior
[lb_prior, ub_prior] = stk_param_getdefaultbounds(prior);

len_param = size(list_param, 2);
if length(lb_prior) ~= size(list_param, 2)
    stk_error(['The dimension of the log-prior and the length of',...
        'the parameters does not fit.'], 'IncorrectSize');
end

%% Get informations of parameters
lb_def = min(list_param, [], 1);    % default bounds
ub_def = max(list_param, [], 1);
nb_def = NaN(len_param, 1);
for klp = 1:len_param
    nb_def(klp) = length(unique(list_param(:, klp)));
    % number of different elements for each parameter columns
end

%% Change bounds and number of parameters
lb = NaN(len_param, 1);
ub = NaN(len_param, 1);
nb = nb_def;

for kd = 1:len_param
    lb_prior_kd = lb_prior(kd);
    ub_prior_kd = ub_prior(kd);
    
    % first case, ub_prior_kd == lb_prior_kd
    if (~isinf(lb_prior_kd) && ~isinf(ub_prior_kd)...
            && stk_isequal_tolrel(lb_prior_kd, ub_prior_kd))...
            || (isinf(lb_prior_kd) && isinf(ub_prior_kd)...
            && lb_prior_kd*ub_prior_kd > 0)
        % Warning : stk_isequal_tolrel(-Inf, +Inf) return true.
        
        lb(kd) = (lb_prior_kd + ub_prior_kd)/2;
        ub(kd) = (lb_prior_kd + ub_prior_kd)/2;
        nb(kd) = 1;
        
    elseif nb(kd) == 1 % only one value for this parameters
        if ~isinf(lb_prior_kd)
            if ~isinf(ub_prior_kd)
                % lb ~= -Inf && ub ~= +Inf
                lb(kd) = (ub_prior_kd + lb_prior_kd)/2;
            else
                % lb ~= -Inf && ub == +Inf
                lb(kd) = max(lb_prior_kd, lb_def(kd));
            end
        else
            if ~isinf(ub_prior_kd)
                % lb == -Inf && ub ~= + Inf
                lb(kd) = min(ub_prior_kd, ub_def(kd));
            else
                % lb == -Inf && ub == +Inf
                lb(kd) = lb_def(kd);    % == ub_def(kd)
            end
        end
        ub(kd) = lb(kd);
        % nb(kd) = 1;

    else % ub ~= lb and nb(kd) > 1 : default behavior
        lb_def_kd = lb_def(kd);
        ub_def_kd = ub_def(kd);
        
        if (ub_prior_kd < lb_def_kd) || (ub_def_kd < lb_prior_kd)
            % Awkward cases : 
            % l_p < u_p < m_d < M_d
            % m_d < M_d < l_p < u_p
            lb(kd) = min(lb_def_kd, lb_prior_kd);
            ub(kd) = max(ub_def_kd, ub_prior_kd);
            nb(kd) = 3*nb(kd);
        else
            % Compatible cases :
            % l_p < m_d <= u_p < M_d
            % m_d < l_p < u_p < M_d
            % l_p < m_d < M_d < u_p
            % m_d < l_p < u_p < M_d
            lb(kd) = max(lb_def_kd, lb_prior_kd);
            ub(kd) = min(ub_def_kd, ub_prior_kd);
            % nb(kd) = nb(kd);
        end
        
    end
end

%% Adapt the bounds, return the list
list_param_adapted = stk_sampling_regulargrid(nb, len_param, [lb'; ub']);
list_param_adapted = list_param_adapted.data;


end

