%STK_PARAM_INIT_ADAPT_TO_PRIOR is used by stk_param_init to transform a
%list of tested parameters to a list which fit better the prior
%distribution.
%
% CALL : list_param_adapted = stk_param_init_adapt_to_prior(gauss_lp, list_param)
% [overload stk_prior function]
function list_param_adapted = stk_param_init_adapt_to_prior(gauss_lp, list_param)

if nargin > 2
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

if nargin < 2
    stk_error('Too few input arguments', 'TooFewInputArgs');
end

%% Check dimension

len_param = size(list_param, 2);
if dimension(gauss_lp) ~= size(list_param, 2)
    stk_error(['The dimension of the Gaussian log-prior and the length of',...
        ' the parameters does not fit.'], 'IncorrectSize');
end

%% Get parameters of the Gaussian log-prior
trans_matrix = gauss_lp.eigenvect;
mean_prior   = gauss_lp.mean;
var_prior    = gauss_lp.eigenvals;
std_prior    = sqrt(var_prior);

list_transform = list_param*trans_matrix;    % nbTry x len_param
% Transform parameters in a space where they are independent

% Bounds at 5 sigma
mean_transform = trans_matrix'*mean_prior;
lb_inde_prior = mean_transform - 5*std_prior;
ub_inde_prior = mean_transform + 5*std_prior;

%% Get informations of parameters
lb_inde_def = min(list_transform, [], 1);    % default bounds
ub_inde_def = max(list_transform, [], 1);
nb_inde_def = NaN(len_param, 1);
for klp = 1:len_param
    nb_inde_def(klp) = length(unique(list_transform(:, klp)));
    % number of different elements for each parameter columns
end

%% Change bounds and number of parameters
lb_inde = NaN(len_param, 1);
ub_inde = NaN(len_param, 1);
nb_inde = nb_inde_def;

for kid = 1:len_param
    % first case, ub_prior_kd == lb_prior_kd
    if var_prior(kid) < 1e-6
        lb_inde(kid) = mean_transform(kid);
        ub_inde(kid) = mean_transform(kid);
        nb_inde(kid) = 1;
        
    elseif nb_inde(kid) == 1 % only one value for this parameters
        if ~isinf(var_prior(kid))
            lb_inde(kid) = mean_transform(kid);
        else
            lb_inde(kid) = lb_inde_def(kid);    % == ub_inde_def(kd)
        end
        ub_inde(kid) = lb_inde(kid);
        % nb_inde(kd) = 1;

    else % ub ~= lb and nb(kd) > 1 : default behavior
        lb_ipri_kd = lb_inde_prior(kid);
        ub_ipri_kd = ub_inde_prior(kid);
        lb_idef_kd = lb_inde_def(kid);
        ub_idef_kd = ub_inde_def(kid);
        
        if (ub_ipri_kd < lb_idef_kd) || (ub_idef_kd < lb_ipri_kd)
            % Awkward cases : 
            % l_p < u_p < m_d < M_d
            % m_d < M_d < l_p < u_p
            lb_inde(kid) = min(lb_idef_kd, lb_ipri_kd);
            ub_inde(kid) = max(ub_idef_kd, ub_ipri_kd);
            nb_inde(kid) = 3*nb_inde(kid);
        else
            % Compatible cases :
            % l_p < m_d <= u_p < M_d
            % m_d < l_p < u_p < M_d
            % l_p < m_d < M_d < u_p
            % m_d < l_p < u_p < M_d
            lb_inde(kid) = max(lb_idef_kd, lb_ipri_kd);
            ub_inde(kid) = min(ub_idef_kd, ub_ipri_kd);
            % nb_inde(kd) = nb_inde(kd);
        end
        
    end
end

%% Adapt the bounds

% Reduce nb_inde if to important
max_nb = 1e6;
while prod(nb_inde) > max_nb
    coeff = prod(nb_inde)/max_nb;
    nb_inde = nb_inde/coeff^(1/len_param);
    nb_inde = max(floor(nb_inde), 1);
end

list_transform_adapted = stk_sampling_regulargrid(nb_inde, len_param, [lb_inde'; ub_inde']);
list_transform_adapted = list_transform_adapted.data;

%% Get back to initial space
list_param_adapted = list_transform_adapted*trans_matrix';
end

