% STK_PRIOR_GAUSS creates a gaussian prior
%
% CALL : GAUSS_LP = stk_prior_gauss(MEAN_VECT, COVAR_MATRIX)
%
%   builds a multivariate gaussian prior with mean vector MEAN_VECT and
%   covariance matrix COVAR_MATRIX. COVAR_MATRIX must be a square positive
%   matrix. The size of COVAR_MATRIX must be the length of MEAN_VECT.
%
% CALL : GAUSS_LP = stk_prior_gauss(MEAN_VECT, VAR_VECT)
%
%   builds a gaussian prior with mean vector MEAN_VECT and variance vector
%   VAR_VECT. MEAN_VECT and VAR_VECT must have the same legnth.
%   Is equivalent to stk_prior_gauss(MEAN_VECT, diag(VAR_VECT)).
%
% CALL : GAUSS_LP = stk_prior_gauss(MEAN_VECT, COV_EIGENVALS, COV_EIGENVECTS)
%
%   does the same thing, but build the covariance matrix as
%       COVAR_MATRIX = COV_EIGENVECTS * diag(COV_EIGENVALS) * COV_EIGENVECTS'.
%   with eye(size(COVAR_MATRIX)) = COV_EIGENVECTS * COV_EIGENVECTS'.
%
% Note
%
%   The object GAUSS_LP saves the eigenvectors and the eigenvalues of the
%   covariance matrix, in order to ensure positiveness, and accelerate
%   inversion matrix.
function gauss_lp = stk_prior_gauss(meanvect, covar_info, eigenvect)

if nargin > 3
    stk_error('Too many input arguments', 'TooManyInputArgs');
end


if nargin < 1
    %% First case : nargin == 0
    meanvect = 0;
    eigenvals = 0;
    eigenvect = 1;
else
    %% Check mean vector
    % Check class
    if ~isnumeric(meanvect)
        try
            meanvect = double(meanvect);
        catch
            stk_error('The mean vector must be a double.',...
                'InvalidArgument');
        end
    end
    
    % Check size
    if ~isvector(meanvect)
        stk_error('The mean vector must be a vector.', 'InvalidArgument');
    end
    
    meanvect = meanvect(:); % assert column vector
    len = length(meanvect);
    
    % No NaN values
    if any(isnan(meanvect))
        stk_error('The mean vector must not have NaN values.',...
            'matrixWithNaN');
    end
    
    % No NaN values
    if any(isinf(meanvect))
        stk_error('The mean vector must not have Inf values.',...
            'matrixWithInf');
    end
    
    %% Check variance vector
    % Default value
    if nargin < 2 || isempty(covar_info)
        covar_info = Inf(len, 1);
    else
        % Check class
        if ~isnumeric(covar_info)
            try
                covar_info = double(covar_info);
            catch
                stk_error('The covariance must be a double.',...
                    'InvalidArgument');
            end
        end
        
        % No NaN values
        if any(isnan(covar_info(:)))
            stk_error('The covariance must not have NaN values.',...
                'matrixWithNaN');
        end
    end
    
    % Find eigen values and eigen vectors
    if size(covar_info, 1) == size(covar_info, 2) % case square matrix
        if nargin > 2 && ~isempty(eigenvect)
            stk_error(['You cannot input at the same time the covariance',...
                ' matrix and its eigen vectors.'], 'InvalidArgument');
        end
        
        % No Inf values
        if any(isinf(covar_info(:)))
            stk_error('The covariance must not have Inf values.',...
                'matrixWithInf');
        end
        [eigenvect, eigenvals] = eig(covar_info);
        eigenvals = diag(eigenvals);
    else
        eigenvals = covar_info;
        % Check vectors
        if ~isvector(eigenvals)
            stk_error('The 2nd argument must be a vector or a square matrix.',...
                'IncorrectSize');
        end
        eigenvals = eigenvals(:);   % assert column vector
        
        if nargin < 3 || isempty(eigenvect)
            eigenvect = eye(len);
        else
            %% Check eigen vectors
            % Check class
            if ~isnumeric(eigenvect)
                try
                    eigenvect = double(eigenvect);
                catch
                    stk_error('The eigen vectors must be a double.',...
                        'InvalidArgument');
                end
            end
            
            % No NaN values
            if any(isnan(eigenvect(:)))
                stk_error('The eigen vectors must not have NaN values.',...
                    'matrixWithNaN');
            end
            % No Inf values
            if any(isinf(eigenvect(:)))
                stk_error('The eigen vectors must not have Inf values.',...
                    'matrixWithInf');
            end
            
            % Check size
            if size(eigenvect, 1) ~= size(eigenvect, 2)
                stk_error('The eigen vectors must form a square matrix.',...
                    'IncorrectSize');
            end
            if size(eigenvect, 1) ~= len
                stk_error(['The eigen vectors must have the same size',...
                    ' as the mean vector.'], 'IncorrectSize');
            end
            
            % Check orthogonality
            if ~stk_isequal_tolabs(eigenvect*eigenvect', eye(len))
                stk_error('The eigen vectors must be orthogonal.',...
                    'InvalidArgument');
            end
        end
    end
    %% Finish to analyse variance vector
    % Compare lengths
    if length(eigenvals) ~= len
        stk_error(['The size of the covariance matrix and the mean vector',...
            ' are incompatible.'], 'IncorrectSize');
    end
    
    % Assert positivity
    if any(eigenvals < 0)
        stk_error('The covariance matrix must be positive.',...
            'InvalidArgument');
    end
    
end % if nargin < 1


%% Build the prior
gauss_lp = struct('mean', meanvect, 'eigenvals', eigenvals, 'eigenvect', eigenvect);
gauss_lp = class(gauss_lp, 'stk_prior_gauss', stk_prior());
end

