% CALL : model = stk_create_model_matern(is_iso, dim, is_regOpt, reg, is_stoc)
%
% Create a model with Matern covariance function.
%
% - is_iso (default value : false) : a boolean. Choose between isotropic and
%       anisotropic Matern;
% - dim (default value : 1) : an integer. Dimension of the model;
% - is_regOpt (default : isnan(reg)) : a boolean. Indicate if the
% regularity should be optimize or not;
% - reg (default value : NaN) : a positive number. Regulatity of the matern
% function;
% - is_stoc (default : false) : a boolean indicating if the model
% is stochastic or not.
%
% Return a model with the new convention.
function model = stk_create_model_matern(is_iso, dim, is_regOpt, reg, is_stoc)

if nargin > 5
    stk_error('Too many input arguments', 'TooManyInputArgs');
end
%% Default value
if nargin < 5 || isempty(is_stoc)
    is_stoc = false;
end

if nargin < 4 || isempty(reg)
    reg = NaN;
end

if nargin < 3 || isempty(is_regOpt)
    is_regOpt = isnan(reg);
elseif isnan(reg) && ~is_regOpt
    stk_error('If regularity is not optimized, a value must be provided.', 'nanValue');
elseif isinf(reg) && is_regOpt
    warning('STK:stk_create_model_matern:infRegularity',...
        'If regularity is infinite, it cannot be optimized.');
end

if nargin < 2 || isempty(dim)
    dim = 1;
end

if nargin < 1 || isempty(is_iso)
    is_iso = false;
end

%% Build a model
if is_iso
    if isinf(reg)
        model = stk_model('stk_gausscov_iso', dim);
    else
        model = stk_model('stk_materncov_iso', dim);
    end
else % is_aniso
    if isinf(reg)
        model = stk_model('stk_gausscov_aniso', dim);
    else
        model = stk_model('stk_materncov_aniso', dim);
    end
end

%% Create covariance model
if is_iso
    covmo = stk_covmodel_materniso(reg);
    lp_dim = 3;
else % is_aniso
    covmo = stk_covmodel_maternaniso(dim, reg);
    lp_dim = dim + 2;
end

if ~is_regOpt && ~isinf(reg)
    mean_vec = [zeros(lp_dim - 1, 1); reallog(reg)];
    var_vec  = [ +Inf(lp_dim - 1, 1); 0];
    prior = stk_prior_gauss(mean_vec, var_vec);
else
    prior = [];
end

covmo.prior = prior;
model.param = covmo;

%% Create noise
if is_stoc
    model.lognoisevariance = nan;
else
    model.lognoisevariance = -inf;
end

end

