function [x_cand, red_cand] = SUR_lev(xc, dt, bnd, model, xi_01, zi, ...
    x_cur_01, xc_int_01, thres, frac_prun, ns_max, do_optim)
% SUR_lev computes the SUR at each level.
%
% CALL: [x_cand, red_cand] = SUR_lev(xc, dt, bnd, model, xi_01, zi, ...
%    x_cur_01, xc_int_01, thres, frac_prun, ns_max, do_optim)
%
%   Candidate points
%   * xc            : candidate points in x (dimension dim - 1)
%   * dt            : list of levels (not normalized)
%   * bnd           : input bounds (dimension dim)
%   Model
%   * model         : prior model
%   * xi_01         : input observations, rescaled in [0; 1]^d
%   * zi            : ouput observations
%   Parameters of the criteria
%   * x_cur_01      : current observation(s), rescaled in [0; 1]^d
%   (dimension: 1 x (q*dim))
%   * xc_int_01     : integration points, rescaled in [0; 1]^d (d = dim)
%   * thres         : critical threshold
%   * frac_prun     : fraction pruning
%   * ns_max        : maximal number of candidate points
%   * do_optim      : use fmincon (if available) to optimize or not
%   (default: true, if fmincon available)
%
%   Outputs
%   * x_cand    : candidate by levels dt (size = 1 x dim, not scaled in [0; 1]^d)
%   * red_cand  : value of the reduction criterion.

if nargin > 12
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

if nargin < 9
    stk_error('Not enough input arguments', 'NotEnoughInputArgs');
end

%%
xc = double(xc);
dt = double(dt);
nb_dt = numel(dt);

dimx = size(xc, 2);
dim  = dimx + 1;

if ~isempty(bnd)
    bnd = stk_hrect(bnd);   % Check bounds
    if size(bnd, 2) ~= dim
        stk_hrect('Incompatible size between bnd and xc', 'IncorrectSize');
    end
else
    bnd = stk_hrect(dim);
end

nb_model = numel(model);
if size(zi, 2) ~= nb_model
    stk_error('Number of model and number of columns in zi are not the same', 'IncorrectSize');
end

if ~isempty(x_cur_01)
    x_cur_01 = double(x_cur_01);
    if size(x_cur_01, 1) > 1
        stk_error('Maximum one current design allowed', 'IncorrectSize');
    end
    
    if mod(size(x_cur_01, 2), dim) > 0
        stk_error('Invalid dimension', 'IncorrectSize');
    end
else % isempty
    x_cur_01 = NaN(1, 0);
end

if size(xc_int_01, 2) ~= dim
    stk_error('Intergration points does not have the right dimension', 'IncorrectSize');
end

thres = double(thres);
if numel(thres) ~= nb_model
    stk_error('Number of model and number of thresholds are not the same', 'IncorrectSize');
end

if nargin < 10 || isempty(frac_prun)
    frac_prun = 0.9;	% Default value: 90%
end

if nargin < 11 || isempty(ns_max)
    ns_max = 1000;      % Default value for numer of select points: 1000
end

if nargin < 12 || isempty(do_optim)
    % By default, try to optimize
    do_optim = true;
end

if do_optim && any(~ismember(xc, bnd(:, 1:dimx)))
    warning('Incompatible bounds and candidate points. xc rescaled');
    for kx = 1:dimx
        % Projection in bnd
        xc(logical(xc(:, kx) < bnd(1, kx)), kx) = bnd(1, kx);
        xc(logical(xc(:, kx) > bnd(2, kx)), kx) = bnd(2, kx);
    end
end

if do_optim
    uncer0 = stk_sur_tot(thres, xc_int_01, model, xi_01, zi, []);
    % the total uncertainty (if nothing is done)
end

% Future results
unc_cand = NaN(nb_dt, 1);
red_cand = NaN(nb_dt, 1);
x_cand = NaN(nb_dt, dim);


% Look for the best points at each levels
for lev = 1:nb_dt
    % fprintf('SUR_lev: %d/%d\n', lev, nb_dt);
    
    % The current level
    dt_lev = dt(lev);
    % Add the level to the candidate points
    xc_lev = stk_add_level(xc, dt_lev);
    % Rescale in [0; 1]^d
    xc_lev_01 = stk_rescale(xc_lev, bnd, stk_hrect(dim));
    
    % Prune the candidate points
    [xc_prun_01, ind_prun] = sur_pruning_tot(thres, xc_lev_01,...
        model, xi_01, zi, frac_prun, ns_max);
    % Non-scaled candidate points
    xc_prun = xc_lev(ind_prun, :);
    
    % Add previous observations
    x_crit_01 = [repmat(x_cur_01, size(xc_prun_01, 1), 1), xc_prun_01];
    % Compute the reduction of uncertainty
    [unc_val, red_val] = stk_sur_tot(thres, xc_int_01, model, xi_01, zi, x_crit_01);
    
    % Select the maximum
    [~, idx_select] = min(unc_val);
    % Save the maximum
    unc_cand(lev, 1) = unc_val(idx_select);
    red_cand(lev, 1) = red_val(idx_select);
    x_cand(lev, :) = xc_prun(idx_select, :);
    
    if do_optim
        % Use fmincon to minimize the function
        dt_lev01 = stk_rescale(dt_lev, bnd(:, dim), [0; 1]);
        
        future_unc = @(x)(sur_tot_pre_lev(x, thres,...
            xc_int_01, model, xi_01, zi, x_cur_01, dt_lev01));
        
        xinit_01 = xc_prun_01(idx_select, 1:dimx);
        % Point should be in [0; 1]^d
        lower_bnd = zeros(1, dimx);
        upper_bnd =  ones(1, dimx);
        
        % Maximum number of evalutations
        maxEval = 20*dimx;
        try
            % try with fmincon (if available)
            % Constraints
            A_in = [];  B_in = [];  A_eq = [];  B_eq = [];  nonlcon = [];
            % Options: semi-Quadratic programming, and no message displayed
            opts_fmincon = optimoptions('fmincon',...
                'Algorithm', 'sqp', 'Display', 'off', 'MaxFunEvals', maxEval);
            % Fmincon
            [xmin_01, unc_min] = fmincon(future_unc, xinit_01,... function and initial point
                A_in, B_in, A_eq, B_eq, lower_bnd, upper_bnd, nonlcon,...Constraints
                opts_fmincon);  % Options
        catch
            % Use Fminsearch
            % Options: do not display any message
            opts_fminsearch = optimset('fminsearch');
            opts_fminsearch = optimset(opts_fminsearch,...
                'Display', 'off', 'MaxFunEvals', maxEval);
            [xmin_01, unc_min] = fminsearch(future_unc, xinit_01,... function and initial point
                opts_fminsearch);   % Options
        end
        
        
        % Do not forget the level
        xmin_01 = stk_add_level(xmin_01, dt_lev01);
        
        if all(xmin_01(:) <= 1) && all(xmin_01(:) >= 0)
            % If the optimum is inside the bounds
            % Rescale optimal value
            xmin = stk_rescale(xmin_01, [[lower_bnd; upper_bnd], [0; 1]], bnd);
            
            % Save new best values
            unc_cand(lev, 1) = unc_min;
            red_cand(lev, 1) = uncer0 - unc_min;
            x_cand(lev, :) = xmin;
        end % else do not change anything
    end
end

end

function [unc_tot, red_tot] = stk_sur_tot(thres, xc_int_01, model, xi_01, zi, x_crit_01)
ncrit = size(x_crit_01, 1);

if ncrit < 1 % Not empty
    unc_tot = 0;
    red_tot = 0;
else
    unc_tot = zeros(ncrit, 1);
    red_tot = zeros(ncrit, 1);
end

nb_model = numel(model);
for km = 1:nb_model
    [unc_val, red_val] = stk_sur_pexceedthres(thres(km), xc_int_01,...
        model(km), xi_01, zi(:, km), x_crit_01);
    
    unc_tot = unc_tot + unc_val;
    red_tot = red_tot + red_val;
end


end

function unc_tot = sur_tot_pre_lev(xcand_01, thres, xc_int_01, model, xi_01, zi, x_cur_01, dt_lev01)

ncand = size(xcand_01, 1);
x_future_01 = [repmat(x_cur_01, ncand, 1), stk_add_level(xcand_01, dt_lev01)];

unc_tot = stk_sur_tot(thres, xc_int_01, model, xi_01, zi, x_future_01);

end

function [xc_prun_01, ind_prun] = sur_pruning_tot(thres, xc_lev_01,...
    model, xi_01, zi, frac_prun, ns_max)

nc = size(xc_lev_01, 1);

nb_model = numel(model);
%% Compute the global uncertainty
unc_zc = zeros(nc, 1);

for km = 1:nb_model
    % First case: MaP parameters
    % Predict
    zc_k = stk_predict(model(km), xi_01, zi(:, km), xc_lev_01);
    if stk_isnoisy(model(km))
        var_obs_k = zc_k.var + stk_covmat_noise (model(km), xc_lev_01, [], -1, true);
        
        % Difference to the threshold
        diff_thres_c = (zc_k.mean - thres(km))./sqrt(var_obs_k);
        % Expectation of the probability of exceeding the threshold
        prob_c = stk_distrib_normal_cdf(diff_thres_c, 0, 1);
        % Variance
        unc_zc_k = stk_distrib_bivnorm_cdf([diff_thres_c, diff_thres_c], 0, 0, 1, 1,...
            zc_k.var./var_obs_k);
        unc_zc_k = unc_zc_k - prob_c.^2;
    else
        % Difference to the threshold
        diff_thres_c = (zc_k.mean - thres(km))./sqrt(zc_k.var);
        [prob_c, quob_c] = stk_distrib_normal_cdf(diff_thres_c, 0, 1);
        unc_zc_k = prob_c.*quob_c;
    end
    
    unc_zc = unc_zc + unc_zc_k;
end

%% Prune the proposed points
% Sort the results
[unc_zc_sort, ind_sort] = sort (unc_zc, 'descend');
if sum(unc_zc_sort) > 0
    cum_unc = cumsum(unc_zc_sort)/sum(unc_zc_sort);
    
    % Select the best
    ns = find(cum_unc >= frac_prun, 1);
    ns = min(ns, ns_max);
    ns = max(ns, 1);
    % 1 <= ns <= ns_max
    ind_prun = ind_sort(1:ns);
else % Case sum(unc_zu_sort) = 0
    ind_prun = randperm(nc, ns_max);
end


% Returns the points xu which are the most uncertain
xc_prun_01 = xc_lev_01(ind_prun, :);
end