% STK_SUR_PRUNING selects the most important points to compute the criterion
%
% CALL: XS = stk_sur_pruning(THRES, XU, MODEL, XI, ZI)
%
%   computes the uncertainty on the probability of exceeding the threshold
%   THRES at the points XU, and return a subset XS which are the most
%   uncertainty. The number of points ns = size(XS, 1) is such that
%       sum(Var[P(XS)]) >= FRAC*sum(Var[P(XU)]),
%   with a maximum NS_MAX = 1000, and FRAC = 90%.
%   MODEL, XI, ZI describes a posterior model with observations (XI, ZI).
%
% CALL: XS = stk_sur_pruning(THRES, XU, MODEL, XI, ZI, WEIGHTS, FRAC,
%       NS_MAX)
%
%   does the same thing, but allows to change the internal parameters.
%   WEIGHTS allows to have a weighted sums; FRAC is the fractional of
%   representativity in the criterion; and NS_MAX is the maximal number of
%   rows of XS.
%   Default values:
%       * WEIGHTS = ones(size(XU, 1), 1);
%       * FRAC = 90%;
%       * NS_MAX = 1000.
%
% CALL: [XS, I_SELECT] = stk_sur_pruning(THRES, XU, MODEL, XI, ZI, WEIGHTS,
%       FRAC, NS_MAX)
%
%   also returns the index such as XS = XU(I_SELECT, :).
%
% CALL: XS = stk_sur_pruning(THRES, XU, MODEL, XI, ZI, WEIGHTS, FRAC,
%       NS_MAX, SIMPARAM)
%
%   does the same thing, but computes the uncertainty on the probability of
%   exceeding a threshold by taking into account on uncertainty parameters.
%   SIMPARAM is a nb x dim_param matrix, such as each row is an
%   hyper-parameter of the model sampled according to the posterior
%   distribution.
%
% See also: stk_sur_pexceedthres
function [xs, i_select] = stk_sur_pruning(thres, xu, model, xi, zi, weights, frac, ns_max, simParam)

if nargin > 9
    stk_error('Too many inputs arguments.', 'TooManyInputArgs');
end

%% Check arguments
thres = double(thres);

xu = double(xu);
xi = double(xi);
zi = double(zi);

dim = size(xu, 2);
nu = size(xu, 1);
if size(xi, 2) ~= dim
    stk_error('xi and xu do not have the same dimension.', 'IncorrectSize');
end

if model.dim ~= dim
    stk_error('The inputs and the model do not have the same dimension.',...
        'IncorrectSize');
end

ni = size(xi, 1);
if ~isequal(size(zi), [ni, 1])
    stk_error(['zi must be a column vector with the same length as',...
        ' size(xi, 1).'], 'IncorrectSize');
end

if nargin < 6 || isempty(weights)
    weights = ones(nu, 1);
else
    weights = double(weights);
    weights = weights(:);	% assert columns vector
    
    if length(weights) == 1
        % Stupid case where there is only one weight. In this case, assume
        % the same weight for all xu
        weights = weights*ones(nu, 1);
    elseif length(weights) ~= nu % && length(weights) ~= 1
        stk_error('The weight vector must have the same length as the integration points.', 'IncorrectSize');
    end
end

if nargin < 7 || isempty(frac)
    frac = 0.9;     % Default value: 90%
end

if nargin < 8 || isempty(ns_max)
    ns_max = 1000;  % Default value for numer of select points: 1000
end

%% Compute the global uncertainty
if nargin < 9 || isempty(simParam)
    % First case: MaP parameters
    % Predict
    zu = stk_predict(model, xi, zi, xu);
    if stk_isnoisy(model)
        var_obs_u = zu.var + stk_covmat_noise (model, xu, [], -1, true);
    else
        var_obs_u = zu.var;
    end
    
    % Difference to the threshold
    diff_thres_u = (zu.mean - thres)./sqrt(var_obs_u);
    % Expectation of the probability of exceeding the threshold
    prob_u = stk_distrib_normal_cdf(diff_thres_u, 0, 1);
    % Variance
    unc_zu = stk_distrib_bivnorm_cdf([diff_thres_u, diff_thres_u], 0, 0, 1, 1,...
        zu.var./var_obs_u);
    unc_zu = unc_zu - prob_u.^2;
    
else
    % Second case: fully Bayesian approach
    prob_u_mean_mean = zeros(nu, 1);
    prob_u_var_mean  = zeros(nu, 1);
    prob_u_mean_var  = zeros(nu, 1);
    for kp = 1:size(simParam, 1)
        % 1: change parameters
        model = stk_set_optimizable_parameters(model, simParam(kp, :));
        
        % 2: predict knowing parameters
        zu_k = stk_predict(model, xi, zi, xu);
        if stk_isnoisy(model)
            var_obs_u_k = zu_k.var + stk_covmat_noise (model, xu, [], -1, true);
        else
            var_obs_u_k = zu_k.var;
        end
        
        % 3: compute posterior values knowing parameters
        diff_thres_u_k = (zu_k.mean - thres)./sqrt(var_obs_u_k);
        prob_u_k = stk_distrib_normal_cdf(diff_thres_u_k, 0, 1);
        unc_zu_k = stk_distrib_bivnorm_cdf([diff_thres_u_k, diff_thres_u_k],...
            0, 0, 1, 1, zu_k.var./var_obs_u_k);
        
        % 4: update statistics
        [prob_u_mean_mean, prob_u_var_mean]...
            = stk_update_recursive(kp, prob_u_mean_mean, prob_u_var_mean, prob_u_k);
        prob_u_mean_var = stk_mean_recursive(kp, prob_u_mean_var , unc_zu_k);
        
    end
    % Sum the uncertainty: Var[P(xu)] = Var[E[P(xu)|Theta]] + E[Var[P(xu)|Theta]]
    unc_zu = prob_u_mean_var + prob_u_var_mean;
end

%% Prune the proposed points
% Sort the results
[unc_zu_sort, ind_sort] = sort (weights .* unc_zu, 'descend');
if sum(unc_zu_sort) > 0
    cum_unc = cumsum(unc_zu_sort)/sum(unc_zu_sort);
    
    % Select the best
    ns = max(min(find(cum_unc >= frac, 1), ns_max), 1);
    % 1 <= ns <= ns_max
    i_select = ind_sort(1:ns);
else % Case sum(unc_zu_sort) = 0
    i_select = randperm(nu, ns_max);
end


% Returns the points xu which are the most uncertain
xs = xu(i_select, :);
end

