% STK_VARIANCE_REDUCTION computes the uncertainty reduction of a
% Gaussian process model.
%
% CALL: RED_XIPXT_XU = stk_variance_reduction (XU, MODEL, XI, XT)
%
%   computes the uncertainty reduction at points XU of a Gaussian process
%   model MODEL (with known hyper-parameters), between the case where the
%   points XI are observed and the case where the points XI and XT are
%   observed:
%
%   Red(XU, XT) = Var( Z(XU)| MODEL; XI, zi )
%                   - Var( Z(XU)| MODEL; [XI; XT], [zi; zt] )
%
%   * XU is a NU x DIM matrix, corresponding to the NU points
%       where the reduction of uncertainty must be computed;
%   * MODEL is a gaussian process model;
%   * XI is a NI x DIM matrix corresponding to the current observations;
%   * XT is a NT x (Q*DIM) matrix, where each row is a successive list
%       of Q future new points
%
%   * RED_XIPXT_XU (reduction _knowing XI Plus XT _at XU) is a NU x NT
%   matrix containing the reduction of uncertainty after observing the
%   points XT
%
% EXAMPLE
%
%   % Create a model
%   dim = 1;
%   model = stk_model ('stk_materncov52_iso', dim);
%   model.param = [2*log(1.0); -log(0.5)];
%   % model.lognoisevariance = 2*log(0.4);
%   % Create an axis
%   xu = stk_sampling_regulargrid (101, dim);
%   % Some observations points
%   xi = [0.0; 0.25; 0.3; 0.5; 0.9];    ni = size(xi, 1);
%   xi_plot = kron(xi, [1; 1; NaN]);
%
%   % Example 1: one new future point (q = 1)
%   xt_1 = stk_sampling_regulargrid (97^dim, dim);
%   red_xiPxt_xu_1 = stk_variance_reduction (xu, model, xi, xt_1);
%   var_reduc_1 = mean(red_xiPxt_xu_1);
%
%   figure; hold on;
%   plot(xt_1, var_reduc_1);
%   plot(xi_plot, kron(max(var_reduc_1)*ones(ni, 1), [0; 1; NaN]), 'k--');
%   xlabel('x_t(1)'); ylabel('S_{x_u} var\_reduc(x_u, x_t(1)) dx_u');
%
%   % Example 2: two new future points (q = 2)
%   xt_2 = stk_sampling_regulargrid (97^(2*dim), 2*dim);
%   red_xiPxt_xu_2 = stk_variance_reduction (xu, model, xi, xt_2);
%   var_reduc_2 = mean(red_xiPxt_xu_2);
%   var_reduc_2 = reshape(var_reduc_2, 97, 97);
%
%   figure;
%   surf(unique(xt_2(:, 1)), unique(xt_2(:, 2)), var_reduc_2);
%   xlabel('x_t(1)'); ylabel('x_t(2)');
%   title('S_{x_u} var\_reduc(x_u, [x_t(1), x_t(2)]) dx_u');
%
% REFERENCES
%
%   [1] Clement Chevalier, Julien Bect, David Ginsbourger, Emmanuel
%       Vazquez, Victor Picheny, and Yann Richet.
%       Fast parallel kriging based stepwise uncertainty reduction with
%       application to the identication of an excursion set.
%       Technometrics, 56(4):455-465, 2014.
function red_xiPxt_xu = stk_variance_reduction(xu, model, xi, xt)

if nargin > 4
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

%% Check inputs arguments

xu = double(xu);
xi = double(xi);
xt = double(xt);

dim = size(xu, 2);
if size(xi, 2) ~= dim
    stk_error('xi and xu do not have the same dimension.', 'IncorrectSize');
end

if model.dim ~= dim
    stk_error('The inputs and the model do not have the same dimension.',...
        'IncorrectSize');
end

q_dim = size(xt, 2);    % q*dim
if q_dim < 1 % q_dim == 0
    q_dim = 0;
elseif mod(q_dim, dim) > 0
    stk_error(['The dimension of xt must be a multiple of the dimension',...
        ' of the problem.'], 'IncorrectSize');
end
q = q_dim/dim;
nt = size(xt, 1);
nu = size(xu, 1);

%% Which algorithm use ?
if q == 0
    % No points are added => no reduction
    red_xiPxt_xu = zeros(nu, 1);
    return;
elseif q > 1
    one_new = false;
    NT = nt*q;
    XT = reshape(xt', dim, NT)';
    % Resize xt, of size, nt x (dim*q), to XT, of size (nt*q) x dim.
    % XT(jq + (it - 1)*q, kd) = xt(it, kd + (jq - 1)*dim)
    %   with : 1 <= it <= nt;   1 <= jq <= q;   1 <= kd <= dim
    
    % Internal parameters for matrix inversion
    max_cond = 1/(4*eps);
else
    % Case q == 1
    one_new = true;
    NT = nt;
    XT = xt;    % the new points are already in the good dimension
end

is_noise = stk_isnoisy(model);

%% Compute the variance of the future mean
% m_{n+1}(xu) | {x_{n+1} = xt(:, 1:dim), x_{n+2} = xt(:, dim +
% (1:dim)), ..., x_{n+q} = xt(:, dim*(q - 1) + (1:dim))}
%   ~ N(m_{n}(xu), nu_{n+1}(x_u))
red_xiPxt_xu = NaN(nu, nt);

% Find the maximum allowed array according to memory
NU_max = 1590;
NU_max = min(NU_max, nu);

% Formulas are compute by NU_max package
index_u = @(p)( ((p - 1)*NU_max + 1):min(NU_max*p, nu) );

for pu = 1:ceil(nu/NU_max)
    xu_p = xu(index_u(pu), :);
    nu_p = size(xu_p, 1);
    
    
    % The limitation of this algorithm is the computation of the covariance a
    % posteriori between xt and xu
    %   [~, ~, ~, cov_n_tu_x_tu] = stk_predict(model, xi, [], [xt; xu]);
    % An empirical experiment gives me the time of this part related to the
    % length of [xt; xu] ;
    %   time (s) = 1.18e-07*length^2 - 5.57e-05*length + 0.112
    %   a = 1.18e-07;   b = -5.57e-05; c = 0.112
    % There is (nt/NT_max) computations of covariance.
    % So the total time to compute all covariances is :
    %   T (s) = (a*(nu + NT_max)^2 + b*(nu + NT_max) + c]*(nt/NT_max)
    %         = (a*(nu^2 + 2*nu*NT_max + NT_max^2) + b*(nu + NT_max) + c]*(nt/NT_max)
    %         = (a*(nu^2/NT_max + 2*nu + NT_max) + b*(nu/NT_max + 1) + c/NT_max]*nt
    %         = ( [a*nu^2 + b*nu + c]/NT_max + 2*a*nu + b + a*NT_max )*nt
    % dT/dNT_max = nt*(a - [a*nu^2 + b*nu + c]/NT_max^2)
    % => { dT/dNT_max = 0 <=> a - [a*nu^2 + b*nu + c]/NT_max^2 = 0
    %                     <=> NT_max = sqrt( [a*nu^2 + b*nu + c]/a)
    %                     <=> NT_max = sqrt(nu^2 + [c + b*nu]/a)
    NT_max = floor(realsqrt(nu_p.^2 + (0.112 - 5.57e-05*nu_p)/1.18e-07));
    % number of points which can be added to compute the covariance matrix
    NT_max = min(NT_max, NT);
    
    nt_max = floor(NT_max/q);
    % Number maximal of block of q points per loop
    
    index_t = @(k)( ((k - 1)*(q*nt_max) + 1):min(k*q*nt_max, NT) );
    % index_t(3) = [2*q*nt + 1, 2*q*nt +2, ..., 3*q*nt]
    
    for kt = 1:ceil(NT/(nt_max*q))
        % Select the points of the kt-th values of the criteria
        xt_k = XT(index_t(kt), :);
        
        % Number of block in the loop (nt_max, except in the last loop)
        nt_k = size(xt_k, 1)/q; % Must be an integer
        
        % Localisation of xt elements in the covariance matrix
        loc_xt = (1:(nt_k*q));
        % Localisation of xu elements in the covariance matrix
        loc_xu = (nt_k*q) + (1:nu_p);
        
        % Predict at xt_k and xu
        [~, ~, ~, cov_n_tKu_x_tKu] = stk_predict(model, xi, [], [xt_k; xu_p]);
        % Covariance between xu and xt_k. DOES NOT contain the impact of noise
        
        cov_n_ut_k = cov_n_tKu_x_tKu(loc_xu, loc_xt); 	% size = [      nu, q*nt_max]
        cov_n_tt_k = cov_n_tKu_x_tKu(loc_xt, loc_xt); 	% size = [q*nt_max, q*nt_max]
        
        if is_noise
            % Add observation noise, if there is some
            noise_t_k = stk_covmat_noise (model, xt_k, [], -1, true);
            cov_n_tt_k = cov_n_tt_k + diag(noise_t_k);
        end
        
        if one_new
            % Easy case : one point is added, and the number of
            % t-points is not the minimum allowed.
            
            var_n_tt_k = diag(cov_n_tt_k);
            
            % Remove the case where the variance on a point is zero, and
            % so, is already known
            invvar_n_tt_k = 1./var_n_tt_k;
            invvar_n_tt_k(isinf(invvar_n_tt_k)) = 0;
            
            % diag(cov_n_TT) = zp_T.var
            red_xiPxt_xu(index_u(pu), index_t(kt)) = (cov_n_ut_k.^2)*diag(invvar_n_tt_k);
            % Only works because this is an expectation when adding 1 point.
            %   size(nu_nPq_u) = [nu, nt]
        else
            % Complex case : q > 1 points are added
            % Elements of the output matrix are computed columns by columns
            for l_kt = 1:nt_k
                % Extract the covariance mztrix assocatied to the i_n
                % design of q points
                idx_l = (1:q) + (l_kt - 1)*q;
                cov_n_tt_k_l = cov_n_tt_k(idx_l, idx_l);
                cov_n_ut_k_l = cov_n_ut_k(    :, idx_l);
                
                % if condest(cov_n_tt_k_l) > 1/eps
                %    % if the covariance matrix is too correlated (case of two
                %    % observations points are too close)
                %    cov_n_tt_k_l = cov_n_tt_k_l + norm(cov_n_tt_k_l, 'fro')*eps*eye(q);
                %    % add a litte bit noise to regularize
                % end
                
                % Compute the associated solution
                idx_sol = nt_max*(kt - 1) + l_kt;
                
                if isequal(cov_n_tt_k_l, zeros(q))
                    % If the covariance matrix is zeros, obsrving these
                    % following points cannot gives any information
                    red_xiPxt_xu(index_u(pu), idx_sol) = 0;
                else
                    % Case where there is a non-zero covariance matrix
                    if condest(cov_n_tt_k_l) > max_cond
                        % If the matrix is bad conditionned (typically when the
                        % same point is proposed q times)
                        eigvect_n_tt_k_l = abs(eig(cov_n_tt_k_l));
                        max_eig = max(eigvect_n_tt_k_l);
                        min_eig = min(eigvect_n_tt_k_l);
                        
                        lambda_reg = (max_eig - max_cond*min_eig)/(max_cond - 1);
                        cov_n_tt_k_l = cov_n_tt_k_l + lambda_reg*eye(q);
                        
                    end
                    
                    red_xiPxt_xu(index_u(pu), idx_sol) = diag(cov_n_ut_k_l*(cov_n_tt_k_l\cov_n_ut_k_l'));
                end
            end
            
        end % if one new point, else
        
    end % for kt
end % for pu

%% To ensure stability (case where var_n_tt_k/cov_n_tt_k_l is zero)
if any(isnan(red_xiPxt_xu(:)))
    warning('STK:stk_variance_reduction:NaNVariancesSetToZero',...
        'NaN values in variance reduction are set to zero');
    red_xiPxt_xu(isnan(red_xiPxt_xu)) = 0;
end
if any(isinf(red_xiPxt_xu(:)))
    warning('STK:stk_variance_reduction:InfVariancesSetToZero',...
        'Infinite values in variance reduction are set to zero');
    red_xiPxt_xu(isinf(red_xiPxt_xu)) = 0;
end

end
